/********************************************************************************
** Form generated from reading UI file 'idxsched.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IDXSCHED_H
#define UI_IDXSCHED_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCommandLinkButton>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_IdxSchedW
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *mainExplainLBL;
    QCommandLinkButton *cronCLB;
    QCommandLinkButton *rtidxCLB;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *IdxSchedW)
    {
        if (IdxSchedW->objectName().isEmpty())
            IdxSchedW->setObjectName("IdxSchedW");
        IdxSchedW->resize(504, 403);
        verticalLayout_2 = new QVBoxLayout(IdxSchedW);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        mainExplainLBL = new QLabel(IdxSchedW);
        mainExplainLBL->setObjectName("mainExplainLBL");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(mainExplainLBL->sizePolicy().hasHeightForWidth());
        mainExplainLBL->setSizePolicy(sizePolicy);
        mainExplainLBL->setTextFormat(Qt::AutoText);
        mainExplainLBL->setWordWrap(true);

        verticalLayout->addWidget(mainExplainLBL);

        cronCLB = new QCommandLinkButton(IdxSchedW);
        cronCLB->setObjectName("cronCLB");
        cronCLB->setAutoDefault(false);

        verticalLayout->addWidget(cronCLB);

        rtidxCLB = new QCommandLinkButton(IdxSchedW);
        rtidxCLB->setObjectName("rtidxCLB");
        rtidxCLB->setAutoDefault(false);

        verticalLayout->addWidget(rtidxCLB);


        verticalLayout_2->addLayout(verticalLayout);

        buttonBox = new QDialogButtonBox(IdxSchedW);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(IdxSchedW);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, IdxSchedW, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, IdxSchedW, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(IdxSchedW);
    } // setupUi

    void retranslateUi(QDialog *IdxSchedW)
    {
        IdxSchedW->setWindowTitle(QCoreApplication::translate("IdxSchedW", "Index scheduling setup", nullptr));
        mainExplainLBL->setText(QCoreApplication::translate("IdxSchedW", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Recoll</span> indexing can run permanently, indexing files as they change, or run at discrete intervals. </p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Reading the manual may help you to decide between these approaches (press F1). </p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-"
                        "left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This tool can help you set up a schedule to automate batch indexing runs, or start real time indexing when you log in (or both, which rarely makes sense). </p></body></html>", nullptr));
        cronCLB->setText(QCoreApplication::translate("IdxSchedW", "Cron scheduling", nullptr));
        cronCLB->setDescription(QCoreApplication::translate("IdxSchedW", "The tool will let you decide at what time indexing should run and will install a crontab entry.", nullptr));
        rtidxCLB->setText(QCoreApplication::translate("IdxSchedW", "Real time indexing start up", nullptr));
        rtidxCLB->setDescription(QCoreApplication::translate("IdxSchedW", "Decide if real time indexing will be started when you log in (only for the default index).", nullptr));
    } // retranslateUi

};

namespace Ui {
    class IdxSchedW: public Ui_IdxSchedW {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IDXSCHED_H
