/****************************************************************************
** Meta object code from reading C++ file 'rclmain_w.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../qtgui/rclmain_w.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'rclmain_w.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7RclMainE_t {};
} // unnamed namespace

template <> constexpr inline auto RclMain::qt_create_metaobjectdata<qt_meta_tag_ZN7RclMainE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RclMain",
        "docSourceChanged",
        "",
        "std::shared_ptr<DocSequence>",
        "stemLangChanged",
        "lang",
        "sortDataChanged",
        "DocSeqSortSpec",
        "resultsReady",
        "searchReset",
        "uiPrefsChanged",
        "fileExit",
        "periodic100",
        "toggleIndexing",
        "startMonitor",
        "bumpIndexing",
        "rebuildIndex",
        "specialIndex",
        "startSearch",
        "std::shared_ptr<Rcl::SearchData>",
        "sdata",
        "issimple",
        "previewClosed",
        "Preview*",
        "w",
        "showAdvSearchDialog",
        "showSpellDialog",
        "showWebcacheDialog",
        "showIndexStatistics",
        "showFragButs",
        "showQLCheatSheet",
        "showSpecIdx",
        "showAboutDialog",
        "showMissingHelpers",
        "showActiveTypes",
        "startOnlineManual",
        "startManual",
        "std::string",
        "showDocHistory",
        "showExtIdxDialog",
        "setSynEnabled",
        "showUIPrefs",
        "showIndexConfig",
        "showPTrans",
        "delPTrans",
        "execIndexConfig",
        "showConfigSwitch",
        "showCronTool",
        "execCronTool",
        "showRTITool",
        "execRTITool",
        "showIndexSched",
        "execIndexSched",
        "setUIPrefs",
        "enableNextPage",
        "enablePrevPage",
        "docExpand",
        "Rcl::Doc",
        "showSubDocs",
        "showSnippets",
        "showActionsSearch",
        "startPreview",
        "docnum",
        "doc",
        "keymods",
        "startNativeViewer",
        "pagenum",
        "term",
        "line",
        "enterHistory",
        "openWith",
        "saveDocToFile",
        "populateSideFilters",
        "RclMain::SideFilterUpdateReason",
        "previewNextInTab",
        "sid",
        "previewPrevInTab",
        "previewExposed",
        "resetSearch",
        "eraseDocHistory",
        "eraseSearchHistory",
        "exportSimpleSearchHistory",
        "saveLastQuery",
        "loadSavedQuery",
        "setStemLang",
        "QAction*",
        "id",
        "adjustPrefsMenu",
        "catgFilter",
        "onFragmentsChanged",
        "initDbOpen",
        "toggleFullScreen",
        "sortByDateAsc",
        "on",
        "sortByDateDesc",
        "showResultsAsTable",
        "onSortDataChanged",
        "onExtSortDataChanged",
        "resultCount",
        "setFilterCtlStyle",
        "stl",
        "showTrayMessage",
        "text",
        "onSetDescription",
        "onNewShortcuts",
        "toggleTable",
        "clearDirFilter",
        "hideToolTip",
        "zoomIn",
        "zoomOut",
        "setFiltSpec",
        "onSSearchTypeChanged",
        "enableSideFilters",
        "enable",
        "updateIdxStatus",
        "onWebcacheDestroyed",
        "onSSTypMenu",
        "act",
        "onSSTypCMB",
        "sideFilterChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'docSourceChanged'
        QtMocHelpers::SignalData<void(std::shared_ptr<DocSequence>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'stemLangChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'sortDataChanged'
        QtMocHelpers::SignalData<void(DocSeqSortSpec)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'resultsReady'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'searchReset'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'uiPrefsChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileExit'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'periodic100'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleIndexing'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startMonitor'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'bumpIndexing'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'rebuildIndex'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'specialIndex'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startSearch'
        QtMocHelpers::SlotData<void(std::shared_ptr<Rcl::SearchData>, bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'previewClosed'
        QtMocHelpers::SlotData<void(Preview *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'showAdvSearchDialog'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showSpellDialog'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showWebcacheDialog'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showIndexStatistics'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showFragButs'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showQLCheatSheet'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showSpecIdx'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showAboutDialog'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showMissingHelpers'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showActiveTypes'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startOnlineManual'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startManual'
        QtMocHelpers::SlotData<void(const std::string &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 37, 2 },
        }}),
        // Slot 'startManual'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showDocHistory'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showExtIdxDialog'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSynEnabled'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'showUIPrefs'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showIndexConfig'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showPTrans'
        QtMocHelpers::SlotData<void(const QString &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'showPTrans'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'delPTrans'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'execIndexConfig'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showConfigSwitch'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showCronTool'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'execCronTool'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showRTITool'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'execRTITool'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showIndexSched'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'execIndexSched'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setUIPrefs'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enableNextPage'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'enablePrevPage'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'docExpand'
        QtMocHelpers::SlotData<void(Rcl::Doc)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 2 },
        }}),
        // Slot 'showSubDocs'
        QtMocHelpers::SlotData<void(Rcl::Doc)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 2 },
        }}),
        // Slot 'showSnippets'
        QtMocHelpers::SlotData<void(Rcl::Doc)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 2 },
        }}),
        // Slot 'showActionsSearch'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startPreview'
        QtMocHelpers::SlotData<void(int, Rcl::Doc, int)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 62 }, { 0x80000000 | 57, 63 }, { QMetaType::Int, 64 },
        }}),
        // Slot 'startPreview'
        QtMocHelpers::SlotData<void(Rcl::Doc)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 2 },
        }}),
        // Slot 'startNativeViewer'
        QtMocHelpers::SlotData<void(Rcl::Doc, int, QString, int, bool)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 2 }, { QMetaType::Int, 66 }, { QMetaType::QString, 67 }, { QMetaType::Int, 68 },
            { QMetaType::Bool, 69 },
        }}),
        // Slot 'startNativeViewer'
        QtMocHelpers::SlotData<void(Rcl::Doc, int, QString, int)>(65, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 57, 2 }, { QMetaType::Int, 66 }, { QMetaType::QString, 67 }, { QMetaType::Int, 68 },
        }}),
        // Slot 'startNativeViewer'
        QtMocHelpers::SlotData<void(Rcl::Doc, int, QString)>(65, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 57, 2 }, { QMetaType::Int, 66 }, { QMetaType::QString, 67 },
        }}),
        // Slot 'startNativeViewer'
        QtMocHelpers::SlotData<void(Rcl::Doc, int)>(65, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 57, 2 }, { QMetaType::Int, 66 },
        }}),
        // Slot 'startNativeViewer'
        QtMocHelpers::SlotData<void(Rcl::Doc)>(65, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 57, 2 },
        }}),
        // Slot 'openWith'
        QtMocHelpers::SlotData<void(Rcl::Doc, std::string)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 2 }, { 0x80000000 | 37, 2 },
        }}),
        // Slot 'saveDocToFile'
        QtMocHelpers::SlotData<void(Rcl::Doc)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 2 },
        }}),
        // Slot 'populateSideFilters'
        QtMocHelpers::SlotData<void(RclMain::SideFilterUpdateReason)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 73, 2 },
        }}),
        // Slot 'previewNextInTab'
        QtMocHelpers::SlotData<void(Preview *, int, int)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 2 }, { QMetaType::Int, 75 }, { QMetaType::Int, 62 },
        }}),
        // Slot 'previewPrevInTab'
        QtMocHelpers::SlotData<void(Preview *, int, int)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 2 }, { QMetaType::Int, 75 }, { QMetaType::Int, 62 },
        }}),
        // Slot 'previewExposed'
        QtMocHelpers::SlotData<void(Preview *, int, int)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 2 }, { QMetaType::Int, 75 }, { QMetaType::Int, 62 },
        }}),
        // Slot 'resetSearch'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'eraseDocHistory'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'eraseSearchHistory'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'exportSimpleSearchHistory'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveLastQuery'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadSavedQuery'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setStemLang'
        QtMocHelpers::SlotData<void(QAction *)>(84, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 85, 86 },
        }}),
        // Slot 'adjustPrefsMenu'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'catgFilter'
        QtMocHelpers::SlotData<void(int)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'catgFilter'
        QtMocHelpers::SlotData<void(QAction *)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 85, 2 },
        }}),
        // Slot 'onFragmentsChanged'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'initDbOpen'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleFullScreen'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sortByDateAsc'
        QtMocHelpers::SlotData<void(bool)>(92, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 93 },
        }}),
        // Slot 'sortByDateDesc'
        QtMocHelpers::SlotData<void(bool)>(94, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 93 },
        }}),
        // Slot 'showResultsAsTable'
        QtMocHelpers::SlotData<void(bool)>(95, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 93 },
        }}),
        // Slot 'onSortDataChanged'
        QtMocHelpers::SlotData<void(DocSeqSortSpec)>(96, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Slot 'onExtSortDataChanged'
        QtMocHelpers::SlotData<void(DocSeqSortSpec)>(97, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Slot 'resultCount'
        QtMocHelpers::SlotData<void(int)>(98, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'setFilterCtlStyle'
        QtMocHelpers::SlotData<void(int)>(99, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 100 },
        }}),
        // Slot 'showTrayMessage'
        QtMocHelpers::SlotData<void(const QString &)>(101, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 102 },
        }}),
        // Slot 'onSetDescription'
        QtMocHelpers::SlotData<void(QString)>(103, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'onNewShortcuts'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleTable'
        QtMocHelpers::SlotData<void()>(105, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearDirFilter'
        QtMocHelpers::SlotData<void()>(106, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hideToolTip'
        QtMocHelpers::SlotData<void()>(107, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void()>(108, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void()>(109, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFiltSpec'
        QtMocHelpers::SlotData<void()>(110, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onSSearchTypeChanged'
        QtMocHelpers::SlotData<void(int)>(111, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'enableSideFilters'
        QtMocHelpers::SlotData<void(bool)>(112, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 113 },
        }}),
        // Slot 'updateIdxStatus'
        QtMocHelpers::SlotData<bool()>(114, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot 'onWebcacheDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(115, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 2 },
        }}),
        // Slot 'onSSTypMenu'
        QtMocHelpers::SlotData<void(QAction *)>(116, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 85, 117 },
        }}),
        // Slot 'onSSTypCMB'
        QtMocHelpers::SlotData<void(int)>(118, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'sideFilterChanged'
        QtMocHelpers::SlotData<void()>(119, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RclMain, qt_meta_tag_ZN7RclMainE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RclMain::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7RclMainE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7RclMainE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7RclMainE_t>.metaTypes,
    nullptr
} };

void RclMain::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RclMain *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->docSourceChanged((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<DocSequence>>>(_a[1]))); break;
        case 1: _t->stemLangChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->sortDataChanged((*reinterpret_cast<std::add_pointer_t<DocSeqSortSpec>>(_a[1]))); break;
        case 3: _t->resultsReady(); break;
        case 4: _t->searchReset(); break;
        case 5: _t->uiPrefsChanged(); break;
        case 6: _t->fileExit(); break;
        case 7: _t->periodic100(); break;
        case 8: _t->toggleIndexing(); break;
        case 9: _t->startMonitor(); break;
        case 10: _t->bumpIndexing(); break;
        case 11: _t->rebuildIndex(); break;
        case 12: _t->specialIndex(); break;
        case 13: _t->startSearch((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<Rcl::SearchData>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 14: _t->previewClosed((*reinterpret_cast<std::add_pointer_t<Preview*>>(_a[1]))); break;
        case 15: _t->showAdvSearchDialog(); break;
        case 16: _t->showSpellDialog(); break;
        case 17: _t->showWebcacheDialog(); break;
        case 18: _t->showIndexStatistics(); break;
        case 19: _t->showFragButs(); break;
        case 20: _t->showQLCheatSheet(); break;
        case 21: _t->showSpecIdx(); break;
        case 22: _t->showAboutDialog(); break;
        case 23: _t->showMissingHelpers(); break;
        case 24: _t->showActiveTypes(); break;
        case 25: _t->startOnlineManual(); break;
        case 26: _t->startManual((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1]))); break;
        case 27: _t->startManual(); break;
        case 28: _t->showDocHistory(); break;
        case 29: _t->showExtIdxDialog(); break;
        case 30: _t->setSynEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->showUIPrefs(); break;
        case 32: _t->showIndexConfig(); break;
        case 33: _t->showPTrans((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 34: _t->showPTrans(); break;
        case 35: _t->delPTrans(); break;
        case 36: _t->execIndexConfig(); break;
        case 37: _t->showConfigSwitch(); break;
        case 38: _t->showCronTool(); break;
        case 39: _t->execCronTool(); break;
        case 40: _t->showRTITool(); break;
        case 41: _t->execRTITool(); break;
        case 42: _t->showIndexSched(); break;
        case 43: _t->execIndexSched(); break;
        case 44: _t->setUIPrefs(); break;
        case 45: _t->enableNextPage((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->enablePrevPage((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 47: _t->docExpand((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 48: _t->showSubDocs((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 49: _t->showSnippets((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 50: _t->showActionsSearch(); break;
        case 51: _t->startPreview((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 52: _t->startPreview((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 53: _t->startNativeViewer((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 54: _t->startNativeViewer((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 55: _t->startNativeViewer((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 56: _t->startNativeViewer((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 57: _t->startNativeViewer((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 58: _t->openWith((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 59: _t->saveDocToFile((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 60: _t->populateSideFilters((*reinterpret_cast<std::add_pointer_t<RclMain::SideFilterUpdateReason>>(_a[1]))); break;
        case 61: _t->previewNextInTab((*reinterpret_cast<std::add_pointer_t<Preview*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 62: _t->previewPrevInTab((*reinterpret_cast<std::add_pointer_t<Preview*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 63: _t->previewExposed((*reinterpret_cast<std::add_pointer_t<Preview*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 64: _t->resetSearch(); break;
        case 65: _t->eraseDocHistory(); break;
        case 66: _t->eraseSearchHistory(); break;
        case 67: _t->exportSimpleSearchHistory(); break;
        case 68: _t->saveLastQuery(); break;
        case 69: _t->loadSavedQuery(); break;
        case 70: _t->setStemLang((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 71: _t->adjustPrefsMenu(); break;
        case 72: _t->catgFilter((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 73: _t->catgFilter((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 74: _t->onFragmentsChanged(); break;
        case 75: _t->initDbOpen(); break;
        case 76: _t->toggleFullScreen(); break;
        case 77: _t->sortByDateAsc((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 78: _t->sortByDateDesc((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 79: _t->showResultsAsTable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 80: _t->onSortDataChanged((*reinterpret_cast<std::add_pointer_t<DocSeqSortSpec>>(_a[1]))); break;
        case 81: _t->onExtSortDataChanged((*reinterpret_cast<std::add_pointer_t<DocSeqSortSpec>>(_a[1]))); break;
        case 82: _t->resultCount((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 83: _t->setFilterCtlStyle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 84: _t->showTrayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 85: _t->onSetDescription((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 86: _t->onNewShortcuts(); break;
        case 87: _t->toggleTable(); break;
        case 88: _t->clearDirFilter(); break;
        case 89: _t->hideToolTip(); break;
        case 90: _t->zoomIn(); break;
        case 91: _t->zoomOut(); break;
        case 92: _t->setFiltSpec(); break;
        case 93: _t->onSSearchTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 94: _t->enableSideFilters((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 95: { bool _r = _t->updateIdxStatus();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 96: _t->onWebcacheDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 97: _t->onSSTypMenu((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 98: _t->onSSTypCMB((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 99: _t->sideFilterChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 70:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 73:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 97:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RclMain::*)(std::shared_ptr<DocSequence> )>(_a, &RclMain::docSourceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RclMain::*)(const QString & )>(_a, &RclMain::stemLangChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RclMain::*)(DocSeqSortSpec )>(_a, &RclMain::sortDataChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RclMain::*)()>(_a, &RclMain::resultsReady, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RclMain::*)()>(_a, &RclMain::searchReset, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (RclMain::*)()>(_a, &RclMain::uiPrefsChanged, 5))
            return;
    }
}

const QMetaObject *RclMain::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RclMain::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7RclMainE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::RclMainBase"))
        return static_cast< Ui::RclMainBase*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int RclMain::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 100)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 100;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 100)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 100;
    }
    return _id;
}

// SIGNAL 0
void RclMain::docSourceChanged(std::shared_ptr<DocSequence> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void RclMain::stemLangChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void RclMain::sortDataChanged(DocSeqSortSpec _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void RclMain::resultsReady()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void RclMain::searchReset()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void RclMain::uiPrefsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
