/****************************************************************************
** Meta object code from reading C++ file 'preview_w.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../qtgui/preview_w.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'preview_w.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15PreviewTextEditE_t {};
} // unnamed namespace

template <> constexpr inline auto PreviewTextEdit::qt_create_metaobjectdata<qt_meta_tag_ZN15PreviewTextEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PreviewTextEdit",
        "displayFields",
        "",
        "displayText",
        "displayImage",
        "print",
        "createPopupMenu",
        "QPoint",
        "pos",
        "onAnchorClicked",
        "QUrl",
        "url",
        "reloadAsPlainText",
        "reloadAsHTML",
        "redisplay"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'displayFields'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayText'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayImage'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'print'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'createPopupMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onAnchorClicked'
        QtMocHelpers::SlotData<void(const QUrl &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'reloadAsPlainText'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reloadAsHTML'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redisplay'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PreviewTextEdit, qt_meta_tag_ZN15PreviewTextEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PreviewTextEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QWebEngineView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PreviewTextEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PreviewTextEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PreviewTextEditE_t>.metaTypes,
    nullptr
} };

void PreviewTextEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PreviewTextEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->displayFields(); break;
        case 1: _t->displayText(); break;
        case 2: _t->displayImage(); break;
        case 3: _t->print(); break;
        case 4: _t->createPopupMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 5: _t->onAnchorClicked((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 6: _t->reloadAsPlainText(); break;
        case 7: _t->reloadAsHTML(); break;
        case 8: _t->redisplay(); break;
        default: ;
        }
    }
}

const QMetaObject *PreviewTextEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreviewTextEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PreviewTextEditE_t>.strings))
        return static_cast<void*>(this);
    return QWebEngineView::qt_metacast(_clname);
}

int PreviewTextEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWebEngineView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7PreviewE_t {};
} // unnamed namespace

template <> constexpr inline auto Preview::qt_create_metaobjectdata<qt_meta_tag_ZN7PreviewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Preview",
        "previewClosed",
        "",
        "Preview*",
        "wordSelect",
        "showNext",
        "w",
        "sid",
        "docnum",
        "showPrev",
        "previewExposed",
        "printCurrentPreviewRequest",
        "saveDocToFile",
        "Rcl::Doc",
        "editRequested",
        "searchTextChanged",
        "text",
        "doSearch",
        "str",
        "next",
        "reverse",
        "wo",
        "nextPressed",
        "prevPressed",
        "currentChanged",
        "closeCurrentTab",
        "closeTab",
        "index",
        "emitShowNext",
        "emitShowPrev",
        "emitSaveDocToFile",
        "emitEditRequested",
        "togglePlainPre",
        "onNewShortcuts",
        "zoomIn",
        "zoomOut",
        "onUiPrefsChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'previewClosed'
        QtMocHelpers::SignalData<void(Preview *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'wordSelect'
        QtMocHelpers::SignalData<void(QString)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'showNext'
        QtMocHelpers::SignalData<void(Preview *, int, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 }, { QMetaType::Int, 7 }, { QMetaType::Int, 8 },
        }}),
        // Signal 'showPrev'
        QtMocHelpers::SignalData<void(Preview *, int, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 }, { QMetaType::Int, 7 }, { QMetaType::Int, 8 },
        }}),
        // Signal 'previewExposed'
        QtMocHelpers::SignalData<void(Preview *, int, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 }, { QMetaType::Int, 7 }, { QMetaType::Int, 8 },
        }}),
        // Signal 'printCurrentPreviewRequest'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saveDocToFile'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Signal 'editRequested'
        QtMocHelpers::SignalData<void(Rcl::Doc)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Slot 'searchTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'doSearch'
        QtMocHelpers::SlotData<void(const QString &, bool, bool, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::Bool, 19 }, { QMetaType::Bool, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'doSearch'
        QtMocHelpers::SlotData<void(const QString &, bool, bool)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::Bool, 19 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'nextPressed'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'prevPressed'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'currentChanged'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'closeCurrentTab'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'closeTab'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'emitShowNext'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'emitShowPrev'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'emitSaveDocToFile'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'emitEditRequested'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'togglePlainPre'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onNewShortcuts'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onUiPrefsChanged'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Preview, qt_meta_tag_ZN7PreviewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Preview::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7PreviewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7PreviewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7PreviewE_t>.metaTypes,
    nullptr
} };

void Preview::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Preview *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->previewClosed((*reinterpret_cast<std::add_pointer_t<Preview*>>(_a[1]))); break;
        case 1: _t->wordSelect((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->showNext((*reinterpret_cast<std::add_pointer_t<Preview*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 3: _t->showPrev((*reinterpret_cast<std::add_pointer_t<Preview*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->previewExposed((*reinterpret_cast<std::add_pointer_t<Preview*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 5: _t->printCurrentPreviewRequest(); break;
        case 6: _t->saveDocToFile((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 7: _t->editRequested((*reinterpret_cast<std::add_pointer_t<Rcl::Doc>>(_a[1]))); break;
        case 8: _t->searchTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->doSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 10: _t->doSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 11: _t->nextPressed(); break;
        case 12: _t->prevPressed(); break;
        case 13: _t->currentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->closeCurrentTab(); break;
        case 15: _t->closeTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->emitShowNext(); break;
        case 17: _t->emitShowPrev(); break;
        case 18: _t->emitSaveDocToFile(); break;
        case 19: _t->emitEditRequested(); break;
        case 20: _t->togglePlainPre(); break;
        case 21: _t->onNewShortcuts(); break;
        case 22: _t->zoomIn(); break;
        case 23: _t->zoomOut(); break;
        case 24: _t->onUiPrefsChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Preview* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Preview* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Preview* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Preview* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Preview::*)(Preview * )>(_a, &Preview::previewClosed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Preview::*)(QString )>(_a, &Preview::wordSelect, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Preview::*)(Preview * , int , int )>(_a, &Preview::showNext, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Preview::*)(Preview * , int , int )>(_a, &Preview::showPrev, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Preview::*)(Preview * , int , int )>(_a, &Preview::previewExposed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Preview::*)()>(_a, &Preview::printCurrentPreviewRequest, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Preview::*)(Rcl::Doc )>(_a, &Preview::saveDocToFile, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Preview::*)(Rcl::Doc )>(_a, &Preview::editRequested, 7))
            return;
    }
}

const QMetaObject *Preview::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Preview::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7PreviewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::Preview"))
        return static_cast< Ui::Preview*>(this);
    return QWidget::qt_metacast(_clname);
}

int Preview::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void Preview::previewClosed(Preview * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Preview::wordSelect(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Preview::showNext(Preview * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void Preview::showPrev(Preview * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void Preview::previewExposed(Preview * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 5
void Preview::printCurrentPreviewRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Preview::saveDocToFile(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Preview::editRequested(Rcl::Doc _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
