/********************************************************************************
** Form generated from reading UI file 'uiprefs.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIPREFS_H
#define UI_UIPREFS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QCommandLinkButton>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_uiPrefsDialogBase
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *tabWidget;
    QWidget *tab_ui;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *vboxLayout1;
    QPushButton *viewActionPB;
    QFrame *line1;
    QCheckBox *singleappCB;
    QHBoxLayout *hboxLayout;
    QLabel *textLabelSST;
    QComboBox *ssearchTypCMB;
    QSpacerItem *horizontalSpacer_12;
    QHBoxLayout *hboxLayout1;
    QLabel *hsLBL;
    QSpinBox *maxHistSizeSB;
    QSpacerItem *horizontalSpacer_5;
    QHBoxLayout *hboxLayout2;
    QLabel *startLBL;
    QCheckBox *initStartAdvCB;
    QCheckBox *startWithFragsToolOpenCB;
    QCheckBox *keepSortCB;
    QHBoxLayout *hboxLayout3;
    QLabel *textLabel20;
    QSpinBox *idxTreeDepthSB;
    QSpacerItem *horizontalSpacer_20;
    QHBoxLayout *hboxLayout4;
    QLabel *sidefilterdateformatLBL;
    QLineEdit *sidefilterdateformatLE;
    QSpacerItem *sidefilterdateformatHSPC;
    QFrame *line2;
    QHBoxLayout *horizontalLayout1;
    QLabel *label1;
    QRadioButton *filterBT_RB;
    QRadioButton *filterCMB_RB;
    QRadioButton *filterMN_RB;
    QHBoxLayout *hboxLayout5;
    QLabel *labeltb;
    QCheckBox *noToolbarsCB;
    QCheckBox *noStatusBarCB;
    QCheckBox *noMenuBarCB;
    QCheckBox *noSSTypCMBCB;
    QCheckBox *noClearSearchCB;
    QCheckBox *noSizeFilterCB;
    QHBoxLayout *hboxLayout6;
    QCheckBox *showTrayIconCB;
    QCheckBox *closeToTrayCB;
    QCheckBox *trayMessagesCB;
    QCheckBox *noBeepsCB;
    QCheckBox *showTempFileWarningCB;
    QHBoxLayout *horizontalLayout11;
    QCheckBox *ssNoCompleteCB;
    QCheckBox *ssearchCompletePassiveCB;
    QCheckBox *ssSearchOnCompleteCB;
    QHBoxLayout *horizontalLayout12;
    QHBoxLayout *hboxLayout7;
    QLabel *ssearchCompleterHistCntLBL;
    QSpinBox *ssearchCompleterHistCntSB;
    QSpacerItem *ssearchCompleterHistCntHSPC;
    QCheckBox *showcompleterhitcountsCB;
    QFrame *line3;
    QHBoxLayout *hboxLayout8;
    QLabel *textLabelqtc;
    QComboBox *qtermStyleCMB;
    QHBoxLayout *hboxLayout9;
    QLabel *wuiscaleLBL;
    QDoubleSpinBox *wholeuiscaleSB;
    QSpacerItem *wholeuiscaleHSPC;
    QHBoxLayout *hboxLayout10;
    QLabel *colorschemeLBL;
    QComboBox *colorschemeCMB;
    QHBoxLayout *hboxLayout11;
    QLabel *tLSTL;
    QPushButton *resetSSPB;
    QPushButton *stylesheetPB;
    QSpacerItem *horizontalSpacer_16;
    QSpacerItem *spacer4;
    QHBoxLayout *hboxLayout12;
    QLabel *uilanguageLBL;
    QComboBox *uilanguageCMB;
    QWidget *tab_reslist;
    QVBoxLayout *vboxLayout2;
    QVBoxLayout *vboxLayout3;
    QHBoxLayout *hboxLayout13;
    QLabel *textLabel1;
    QSpinBox *pageLenSB;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *hboxLayout14;
    QLabel *textLabel4;
    QPushButton *reslistFontPB;
    QPushButton *resetFontPB;
    QSpacerItem *horizontalSpacer_13;
    QHBoxLayout *hboxLayout15;
    QCommandLinkButton *CLEditPara;
    QCommandLinkButton *CLEditHeader;
    QHBoxLayout *hboxLayout16;
    QLabel *textLabelDateFormat;
    QLineEdit *dateformatLE;
    QHBoxLayout *hboxLayout17;
    QLabel *textLabelAbss;
    QLineEdit *abssepLE;
    QSpacerItem *horizontalSpacer_14;
    QCheckBox *alwaysSnippetsCB;
    QLabel *tabptr1;
    QFrame *line4;
    QHBoxLayout *hboxLayout18;
    QLabel *tLSTL1;
    QPushButton *snipCssPB;
    QPushButton *resetSnipCssPB;
    QSpacerItem *horizontalSpacer_15;
    QHBoxLayout *hboxLayout19;
    QLabel *snipwMaxLenLBL;
    QSpinBox *snipwMaxLenSB;
    QSpacerItem *horizontalSpacer_3;
    QCheckBox *snipwByPageCB;
    QSpacerItem *verticalSpacer;
    QWidget *tab_restable;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *noResTableHeaderCB;
    QCheckBox *showResTableVHeaderCB;
    QCheckBox *noRowJumpShortcutsCB;
    QHBoxLayout *horizontalLayout;
    QLabel *restabShowTxtLBL;
    QRadioButton *restabShowTxtNoShiftRB;
    QRadioButton *restabShowTxtShiftRB;
    QCheckBox *resTableNoHoverMetaCB;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_preview;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *hboxLayout20;
    QLabel *textLabelHL;
    QSpinBox *maxHLTSB;
    QSpacerItem *horizontalSpacer_4;
    QCheckBox *previewHtmlCB;
    QCheckBox *previewdarkbgCB;
    QCheckBox *previewActiveLinksCB;
    QHBoxLayout *hboxLayout21;
    QLabel *pvmaxfldlenLBL;
    QSpinBox *pvmaxfldlenSB;
    QSpacerItem *pvmaxfldlenHSPC;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label;
    QRadioButton *plainBRRB;
    QRadioButton *plainPRERB;
    QRadioButton *plainPREWRAPRB;
    QHBoxLayout *hboxLayout22;
    QLabel *previewLinesOverAnchorLBL;
    QSpinBox *previewLinesOverAnchorSB;
    QSpacerItem *previewLinesOverAnchorHSPC;
    QSpacerItem *verticalSpacer_3;
    QWidget *tab_shortcuts;
    QVBoxLayout *verticalLayout_4;
    QVBoxLayout *verticalLayout;
    QTableWidget *shortcutsTB;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *resetscPB;
    QWidget *tab_search;
    QVBoxLayout *vboxLayout4;
    QCheckBox *collapseDupsCB;
    QVBoxLayout *vboxLayout5;
    QHBoxLayout *hboxLayout23;
    QLabel *textLabel1_2;
    QComboBox *stemLangCMB;
    QSpacerItem *horizontalSpacer_11;
    QFrame *line5;
    QCheckBox *autoPhraseCB;
    QHBoxLayout *hboxLayout24;
    QLabel *textLabel33;
    QDoubleSpinBox *autoPThreshSB;
    QSpacerItem *horizontalSpacer_6;
    QFrame *line6;
    QCheckBox *buildAbsCB;
    QCheckBox *replAbsCB;
    QHBoxLayout *hboxLayout25;
    QLabel *textLabel2;
    QSpinBox *syntlenSB;
    QSpacerItem *horizontalSpacer_7;
    QHBoxLayout *hboxLayout26;
    QLabel *textLabel3;
    QSpinBox *syntctxSB;
    QSpacerItem *horizontalSpacer_8;
    QFrame *line7;
    QHBoxLayout *hboxLayout27;
    QLabel *textLabel14;
    QCheckBox *autoSuffsCB;
    QLineEdit *autoSuffsLE;
    QSpacerItem *horizontalSpacer_9;
    QHBoxLayout *hboxLayout28;
    QCheckBox *autoSpellCB;
    QLabel *autoSpellLBL;
    QSpinBox *autoSpellMaxDistSB;
    QSpacerItem *horizontalSpacer_91;
    QHBoxLayout *hboxLayout29;
    QLabel *textLabelSG;
    QCheckBox *synFileCB;
    QPushButton *synFilePB;
    QSpacerItem *horizontalSpacer_10;
    QCheckBox *ignwildsCB;
    QSpacerItem *spacer2;
    QWidget *tab_extradbs;
    QVBoxLayout *vboxLayout6;
    QVBoxLayout *vboxLayout7;
    QListWidget *idxLV;
    QHBoxLayout *hboxLayout30;
    QPushButton *togExtraDbPB;
    QPushButton *actAllExtraDbPB;
    QPushButton *unacAllExtraDbPB;
    QPushButton *ptransPB;
    QFrame *line9;
    QHBoxLayout *hboxLayout31;
    QPushButton *delExtraDbPB;
    QSpacerItem *spacer3;
    QPushButton *addExtraDbPB;
    QWidget *tab_misc;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *anchorTamilHackCB;
    QSpacerItem *verticalSpacer_4;
    QHBoxLayout *hboxLayout32;
    QSpacerItem *Horizontal_Spacing2;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;
    QButtonGroup *buttonGroup_2;
    QButtonGroup *buttonGroup;

    void setupUi(QDialog *uiPrefsDialogBase)
    {
        if (uiPrefsDialogBase->objectName().isEmpty())
            uiPrefsDialogBase->setObjectName("uiPrefsDialogBase");
        uiPrefsDialogBase->resize(937, 750);
        uiPrefsDialogBase->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(uiPrefsDialogBase);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        tabWidget = new QTabWidget(uiPrefsDialogBase);
        tabWidget->setObjectName("tabWidget");
        tab_ui = new QWidget();
        tab_ui->setObjectName("tab_ui");
        verticalLayout_3 = new QVBoxLayout(tab_ui);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        scrollArea = new QScrollArea(tab_ui);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 881, 712));
        vboxLayout1 = new QVBoxLayout(scrollAreaWidgetContents);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        viewActionPB = new QPushButton(scrollAreaWidgetContents);
        viewActionPB->setObjectName("viewActionPB");

        vboxLayout1->addWidget(viewActionPB);

        line1 = new QFrame(scrollAreaWidgetContents);
        line1->setObjectName("line1");
        line1->setFrameShape(QFrame::Shape::HLine);
        line1->setFrameShadow(QFrame::Shadow::Sunken);

        vboxLayout1->addWidget(line1);

        singleappCB = new QCheckBox(scrollAreaWidgetContents);
        singleappCB->setObjectName("singleappCB");
        singleappCB->setChecked(false);

        vboxLayout1->addWidget(singleappCB);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        textLabelSST = new QLabel(scrollAreaWidgetContents);
        textLabelSST->setObjectName("textLabelSST");
        textLabelSST->setWordWrap(false);

        hboxLayout->addWidget(textLabelSST);

        ssearchTypCMB = new QComboBox(scrollAreaWidgetContents);
        ssearchTypCMB->setObjectName("ssearchTypCMB");

        hboxLayout->addWidget(ssearchTypCMB);

        horizontalSpacer_12 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(horizontalSpacer_12);


        vboxLayout1->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        hsLBL = new QLabel(scrollAreaWidgetContents);
        hsLBL->setObjectName("hsLBL");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(hsLBL->sizePolicy().hasHeightForWidth());
        hsLBL->setSizePolicy(sizePolicy);
        hsLBL->setWordWrap(false);

        hboxLayout1->addWidget(hsLBL);

        maxHistSizeSB = new QSpinBox(scrollAreaWidgetContents);
        maxHistSizeSB->setObjectName("maxHistSizeSB");
        maxHistSizeSB->setMinimum(-1);
        maxHistSizeSB->setMaximum(1000);
        maxHistSizeSB->setValue(0);

        hboxLayout1->addWidget(maxHistSizeSB);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(horizontalSpacer_5);

        hboxLayout1->setStretch(2, 1);

        vboxLayout1->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        startLBL = new QLabel(scrollAreaWidgetContents);
        startLBL->setObjectName("startLBL");
        sizePolicy.setHeightForWidth(startLBL->sizePolicy().hasHeightForWidth());
        startLBL->setSizePolicy(sizePolicy);
        startLBL->setWordWrap(false);

        hboxLayout2->addWidget(startLBL);

        initStartAdvCB = new QCheckBox(scrollAreaWidgetContents);
        initStartAdvCB->setObjectName("initStartAdvCB");
        initStartAdvCB->setChecked(false);

        hboxLayout2->addWidget(initStartAdvCB);

        startWithFragsToolOpenCB = new QCheckBox(scrollAreaWidgetContents);
        startWithFragsToolOpenCB->setObjectName("startWithFragsToolOpenCB");
        startWithFragsToolOpenCB->setChecked(false);

        hboxLayout2->addWidget(startWithFragsToolOpenCB);

        hboxLayout2->setStretch(1, 1);
        hboxLayout2->setStretch(2, 1);

        vboxLayout1->addLayout(hboxLayout2);

        keepSortCB = new QCheckBox(scrollAreaWidgetContents);
        keepSortCB->setObjectName("keepSortCB");
        keepSortCB->setChecked(false);

        vboxLayout1->addWidget(keepSortCB);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName("hboxLayout3");
        textLabel20 = new QLabel(scrollAreaWidgetContents);
        textLabel20->setObjectName("textLabel20");
        sizePolicy.setHeightForWidth(textLabel20->sizePolicy().hasHeightForWidth());
        textLabel20->setSizePolicy(sizePolicy);
        textLabel20->setWordWrap(false);

        hboxLayout3->addWidget(textLabel20);

        idxTreeDepthSB = new QSpinBox(scrollAreaWidgetContents);
        idxTreeDepthSB->setObjectName("idxTreeDepthSB");
        idxTreeDepthSB->setMinimum(0);
        idxTreeDepthSB->setMaximum(100);
        idxTreeDepthSB->setValue(2);

        hboxLayout3->addWidget(idxTreeDepthSB);

        horizontalSpacer_20 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout3->addItem(horizontalSpacer_20);

        hboxLayout3->setStretch(2, 2);

        vboxLayout1->addLayout(hboxLayout3);

        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setSpacing(6);
        hboxLayout4->setObjectName("hboxLayout4");
        sidefilterdateformatLBL = new QLabel(scrollAreaWidgetContents);
        sidefilterdateformatLBL->setObjectName("sidefilterdateformatLBL");
        sidefilterdateformatLBL->setWordWrap(false);

        hboxLayout4->addWidget(sidefilterdateformatLBL);

        sidefilterdateformatLE = new QLineEdit(scrollAreaWidgetContents);
        sidefilterdateformatLE->setObjectName("sidefilterdateformatLE");
        sidefilterdateformatLE->setMinimumSize(QSize(30, 0));

        hboxLayout4->addWidget(sidefilterdateformatLE);

        sidefilterdateformatHSPC = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout4->addItem(sidefilterdateformatHSPC);


        vboxLayout1->addLayout(hboxLayout4);

        line2 = new QFrame(scrollAreaWidgetContents);
        line2->setObjectName("line2");
        line2->setFrameShape(QFrame::Shape::HLine);
        line2->setFrameShadow(QFrame::Shadow::Sunken);

        vboxLayout1->addWidget(line2);

        horizontalLayout1 = new QHBoxLayout();
        horizontalLayout1->setSpacing(6);
        horizontalLayout1->setObjectName("horizontalLayout1");
        label1 = new QLabel(scrollAreaWidgetContents);
        label1->setObjectName("label1");

        horizontalLayout1->addWidget(label1);

        filterBT_RB = new QRadioButton(scrollAreaWidgetContents);
        buttonGroup_2 = new QButtonGroup(uiPrefsDialogBase);
        buttonGroup_2->setObjectName("buttonGroup_2");
        buttonGroup_2->addButton(filterBT_RB);
        filterBT_RB->setObjectName("filterBT_RB");

        horizontalLayout1->addWidget(filterBT_RB);

        filterCMB_RB = new QRadioButton(scrollAreaWidgetContents);
        buttonGroup_2->addButton(filterCMB_RB);
        filterCMB_RB->setObjectName("filterCMB_RB");

        horizontalLayout1->addWidget(filterCMB_RB);

        filterMN_RB = new QRadioButton(scrollAreaWidgetContents);
        buttonGroup_2->addButton(filterMN_RB);
        filterMN_RB->setObjectName("filterMN_RB");

        horizontalLayout1->addWidget(filterMN_RB);


        vboxLayout1->addLayout(horizontalLayout1);

        hboxLayout5 = new QHBoxLayout();
        hboxLayout5->setSpacing(6);
        hboxLayout5->setObjectName("hboxLayout5");
        labeltb = new QLabel(scrollAreaWidgetContents);
        labeltb->setObjectName("labeltb");

        hboxLayout5->addWidget(labeltb);

        noToolbarsCB = new QCheckBox(scrollAreaWidgetContents);
        noToolbarsCB->setObjectName("noToolbarsCB");
        noToolbarsCB->setChecked(false);

        hboxLayout5->addWidget(noToolbarsCB);

        noStatusBarCB = new QCheckBox(scrollAreaWidgetContents);
        noStatusBarCB->setObjectName("noStatusBarCB");
        noStatusBarCB->setChecked(false);

        hboxLayout5->addWidget(noStatusBarCB);

        noMenuBarCB = new QCheckBox(scrollAreaWidgetContents);
        noMenuBarCB->setObjectName("noMenuBarCB");
        noMenuBarCB->setChecked(false);

        hboxLayout5->addWidget(noMenuBarCB);

        noSSTypCMBCB = new QCheckBox(scrollAreaWidgetContents);
        noSSTypCMBCB->setObjectName("noSSTypCMBCB");
        noSSTypCMBCB->setChecked(false);

        hboxLayout5->addWidget(noSSTypCMBCB);

        noClearSearchCB = new QCheckBox(scrollAreaWidgetContents);
        noClearSearchCB->setObjectName("noClearSearchCB");
        noClearSearchCB->setChecked(false);

        hboxLayout5->addWidget(noClearSearchCB);

        noSizeFilterCB = new QCheckBox(scrollAreaWidgetContents);
        noSizeFilterCB->setObjectName("noSizeFilterCB");
        noSizeFilterCB->setChecked(false);

        hboxLayout5->addWidget(noSizeFilterCB);


        vboxLayout1->addLayout(hboxLayout5);

        hboxLayout6 = new QHBoxLayout();
        hboxLayout6->setSpacing(6);
        hboxLayout6->setObjectName("hboxLayout6");
        showTrayIconCB = new QCheckBox(scrollAreaWidgetContents);
        showTrayIconCB->setObjectName("showTrayIconCB");
        showTrayIconCB->setChecked(false);

        hboxLayout6->addWidget(showTrayIconCB);

        closeToTrayCB = new QCheckBox(scrollAreaWidgetContents);
        closeToTrayCB->setObjectName("closeToTrayCB");
        closeToTrayCB->setChecked(false);

        hboxLayout6->addWidget(closeToTrayCB);

        trayMessagesCB = new QCheckBox(scrollAreaWidgetContents);
        trayMessagesCB->setObjectName("trayMessagesCB");
        trayMessagesCB->setChecked(false);

        hboxLayout6->addWidget(trayMessagesCB);


        vboxLayout1->addLayout(hboxLayout6);

        noBeepsCB = new QCheckBox(scrollAreaWidgetContents);
        noBeepsCB->setObjectName("noBeepsCB");
        noBeepsCB->setChecked(false);

        vboxLayout1->addWidget(noBeepsCB);

        showTempFileWarningCB = new QCheckBox(scrollAreaWidgetContents);
        showTempFileWarningCB->setObjectName("showTempFileWarningCB");
        showTempFileWarningCB->setChecked(true);

        vboxLayout1->addWidget(showTempFileWarningCB);

        horizontalLayout11 = new QHBoxLayout();
        horizontalLayout11->setSpacing(6);
        horizontalLayout11->setObjectName("horizontalLayout11");
        ssNoCompleteCB = new QCheckBox(scrollAreaWidgetContents);
        ssNoCompleteCB->setObjectName("ssNoCompleteCB");
        ssNoCompleteCB->setChecked(false);

        horizontalLayout11->addWidget(ssNoCompleteCB);

        ssearchCompletePassiveCB = new QCheckBox(scrollAreaWidgetContents);
        ssearchCompletePassiveCB->setObjectName("ssearchCompletePassiveCB");
        ssearchCompletePassiveCB->setChecked(false);

        horizontalLayout11->addWidget(ssearchCompletePassiveCB);


        vboxLayout1->addLayout(horizontalLayout11);

        ssSearchOnCompleteCB = new QCheckBox(scrollAreaWidgetContents);
        ssSearchOnCompleteCB->setObjectName("ssSearchOnCompleteCB");
        ssSearchOnCompleteCB->setChecked(true);

        vboxLayout1->addWidget(ssSearchOnCompleteCB);

        horizontalLayout12 = new QHBoxLayout();
        horizontalLayout12->setSpacing(6);
        horizontalLayout12->setObjectName("horizontalLayout12");
        hboxLayout7 = new QHBoxLayout();
        hboxLayout7->setSpacing(6);
        hboxLayout7->setObjectName("hboxLayout7");
        ssearchCompleterHistCntLBL = new QLabel(scrollAreaWidgetContents);
        ssearchCompleterHistCntLBL->setObjectName("ssearchCompleterHistCntLBL");
        sizePolicy.setHeightForWidth(ssearchCompleterHistCntLBL->sizePolicy().hasHeightForWidth());
        ssearchCompleterHistCntLBL->setSizePolicy(sizePolicy);
        ssearchCompleterHistCntLBL->setWordWrap(false);

        hboxLayout7->addWidget(ssearchCompleterHistCntLBL);

        ssearchCompleterHistCntSB = new QSpinBox(scrollAreaWidgetContents);
        ssearchCompleterHistCntSB->setObjectName("ssearchCompleterHistCntSB");
        ssearchCompleterHistCntSB->setMinimum(0);
        ssearchCompleterHistCntSB->setMaximum(100);
        ssearchCompleterHistCntSB->setValue(10);

        hboxLayout7->addWidget(ssearchCompleterHistCntSB);

        ssearchCompleterHistCntHSPC = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout7->addItem(ssearchCompleterHistCntHSPC);

        hboxLayout7->setStretch(2, 1);

        horizontalLayout12->addLayout(hboxLayout7);

        showcompleterhitcountsCB = new QCheckBox(scrollAreaWidgetContents);
        showcompleterhitcountsCB->setObjectName("showcompleterhitcountsCB");
        showcompleterhitcountsCB->setChecked(false);

        horizontalLayout12->addWidget(showcompleterhitcountsCB);


        vboxLayout1->addLayout(horizontalLayout12);

        line3 = new QFrame(scrollAreaWidgetContents);
        line3->setObjectName("line3");
        line3->setFrameShape(QFrame::Shape::HLine);
        line3->setFrameShadow(QFrame::Shadow::Sunken);

        vboxLayout1->addWidget(line3);

        hboxLayout8 = new QHBoxLayout();
        hboxLayout8->setSpacing(6);
        hboxLayout8->setObjectName("hboxLayout8");
        textLabelqtc = new QLabel(scrollAreaWidgetContents);
        textLabelqtc->setObjectName("textLabelqtc");
        textLabelqtc->setWordWrap(false);

        hboxLayout8->addWidget(textLabelqtc);

        qtermStyleCMB = new QComboBox(scrollAreaWidgetContents);
        qtermStyleCMB->setObjectName("qtermStyleCMB");
        qtermStyleCMB->setMinimumSize(QSize(50, 0));
        qtermStyleCMB->setEditable(true);
        qtermStyleCMB->setInsertPolicy(QComboBox::NoInsert);

        hboxLayout8->addWidget(qtermStyleCMB);


        vboxLayout1->addLayout(hboxLayout8);

        hboxLayout9 = new QHBoxLayout();
        hboxLayout9->setSpacing(6);
        hboxLayout9->setObjectName("hboxLayout9");
        wuiscaleLBL = new QLabel(scrollAreaWidgetContents);
        wuiscaleLBL->setObjectName("wuiscaleLBL");
        sizePolicy.setHeightForWidth(wuiscaleLBL->sizePolicy().hasHeightForWidth());
        wuiscaleLBL->setSizePolicy(sizePolicy);
        wuiscaleLBL->setWordWrap(false);

        hboxLayout9->addWidget(wuiscaleLBL);

        wholeuiscaleSB = new QDoubleSpinBox(scrollAreaWidgetContents);
        wholeuiscaleSB->setObjectName("wholeuiscaleSB");
        wholeuiscaleSB->setSingleStep(0.100000000000000);
        wholeuiscaleSB->setValue(1.000000000000000);

        hboxLayout9->addWidget(wholeuiscaleSB);

        wholeuiscaleHSPC = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout9->addItem(wholeuiscaleHSPC);

        hboxLayout9->setStretch(2, 1);

        vboxLayout1->addLayout(hboxLayout9);

        hboxLayout10 = new QHBoxLayout();
        hboxLayout10->setSpacing(6);
        hboxLayout10->setObjectName("hboxLayout10");
        colorschemeLBL = new QLabel(scrollAreaWidgetContents);
        colorschemeLBL->setObjectName("colorschemeLBL");
        colorschemeLBL->setWordWrap(false);

        hboxLayout10->addWidget(colorschemeLBL);

        colorschemeCMB = new QComboBox(scrollAreaWidgetContents);
        colorschemeCMB->setObjectName("colorschemeCMB");
        colorschemeCMB->setMinimumSize(QSize(10, 0));
        colorschemeCMB->setEditable(false);
        colorschemeCMB->setInsertPolicy(QComboBox::NoInsert);

        hboxLayout10->addWidget(colorschemeCMB);


        vboxLayout1->addLayout(hboxLayout10);

        hboxLayout11 = new QHBoxLayout();
        hboxLayout11->setSpacing(6);
        hboxLayout11->setObjectName("hboxLayout11");
        tLSTL = new QLabel(scrollAreaWidgetContents);
        tLSTL->setObjectName("tLSTL");
        tLSTL->setWordWrap(false);

        hboxLayout11->addWidget(tLSTL);

        resetSSPB = new QPushButton(scrollAreaWidgetContents);
        resetSSPB->setObjectName("resetSSPB");

        hboxLayout11->addWidget(resetSSPB);

        stylesheetPB = new QPushButton(scrollAreaWidgetContents);
        stylesheetPB->setObjectName("stylesheetPB");

        hboxLayout11->addWidget(stylesheetPB);

        horizontalSpacer_16 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout11->addItem(horizontalSpacer_16);


        vboxLayout1->addLayout(hboxLayout11);

        spacer4 = new QSpacerItem(20, 70, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacer4);

        hboxLayout12 = new QHBoxLayout();
        hboxLayout12->setSpacing(6);
        hboxLayout12->setObjectName("hboxLayout12");
        uilanguageLBL = new QLabel(scrollAreaWidgetContents);
        uilanguageLBL->setObjectName("uilanguageLBL");
        uilanguageLBL->setWordWrap(false);

        hboxLayout12->addWidget(uilanguageLBL);

        uilanguageCMB = new QComboBox(scrollAreaWidgetContents);
        uilanguageCMB->setObjectName("uilanguageCMB");
        uilanguageCMB->setMinimumSize(QSize(10, 0));
        uilanguageCMB->setEditable(false);
        uilanguageCMB->setInsertPolicy(QComboBox::NoInsert);

        hboxLayout12->addWidget(uilanguageCMB);


        vboxLayout1->addLayout(hboxLayout12);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);

        tabWidget->addTab(tab_ui, QString());
        tab_reslist = new QWidget();
        tab_reslist->setObjectName("tab_reslist");
        vboxLayout2 = new QVBoxLayout(tab_reslist);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(6);
        vboxLayout3->setObjectName("vboxLayout3");
        hboxLayout13 = new QHBoxLayout();
        hboxLayout13->setSpacing(6);
        hboxLayout13->setObjectName("hboxLayout13");
        textLabel1 = new QLabel(tab_reslist);
        textLabel1->setObjectName("textLabel1");
        sizePolicy.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy);
        textLabel1->setWordWrap(false);

        hboxLayout13->addWidget(textLabel1);

        pageLenSB = new QSpinBox(tab_reslist);
        pageLenSB->setObjectName("pageLenSB");
        pageLenSB->setMinimum(1);
        pageLenSB->setMaximum(9999);
        pageLenSB->setValue(8);

        hboxLayout13->addWidget(pageLenSB);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout13->addItem(horizontalSpacer_2);

        hboxLayout13->setStretch(2, 1);

        vboxLayout3->addLayout(hboxLayout13);

        hboxLayout14 = new QHBoxLayout();
        hboxLayout14->setSpacing(6);
        hboxLayout14->setObjectName("hboxLayout14");
        textLabel4 = new QLabel(tab_reslist);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        hboxLayout14->addWidget(textLabel4);

        reslistFontPB = new QPushButton(tab_reslist);
        reslistFontPB->setObjectName("reslistFontPB");

        hboxLayout14->addWidget(reslistFontPB);

        resetFontPB = new QPushButton(tab_reslist);
        resetFontPB->setObjectName("resetFontPB");

        hboxLayout14->addWidget(resetFontPB);

        horizontalSpacer_13 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout14->addItem(horizontalSpacer_13);


        vboxLayout3->addLayout(hboxLayout14);

        hboxLayout15 = new QHBoxLayout();
        hboxLayout15->setSpacing(6);
        hboxLayout15->setObjectName("hboxLayout15");
        CLEditPara = new QCommandLinkButton(tab_reslist);
        CLEditPara->setObjectName("CLEditPara");

        hboxLayout15->addWidget(CLEditPara);


        vboxLayout3->addLayout(hboxLayout15);

        CLEditHeader = new QCommandLinkButton(tab_reslist);
        CLEditHeader->setObjectName("CLEditHeader");

        vboxLayout3->addWidget(CLEditHeader);

        hboxLayout16 = new QHBoxLayout();
        hboxLayout16->setSpacing(6);
        hboxLayout16->setObjectName("hboxLayout16");
        textLabelDateFormat = new QLabel(tab_reslist);
        textLabelDateFormat->setObjectName("textLabelDateFormat");
        textLabelDateFormat->setWordWrap(false);

        hboxLayout16->addWidget(textLabelDateFormat);

        dateformatLE = new QLineEdit(tab_reslist);
        dateformatLE->setObjectName("dateformatLE");
        dateformatLE->setMinimumSize(QSize(30, 0));

        hboxLayout16->addWidget(dateformatLE);


        vboxLayout3->addLayout(hboxLayout16);

        hboxLayout17 = new QHBoxLayout();
        hboxLayout17->setSpacing(6);
        hboxLayout17->setObjectName("hboxLayout17");
        textLabelAbss = new QLabel(tab_reslist);
        textLabelAbss->setObjectName("textLabelAbss");
        textLabelAbss->setWordWrap(false);

        hboxLayout17->addWidget(textLabelAbss);

        abssepLE = new QLineEdit(tab_reslist);
        abssepLE->setObjectName("abssepLE");
        abssepLE->setMinimumSize(QSize(30, 0));

        hboxLayout17->addWidget(abssepLE);

        horizontalSpacer_14 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout17->addItem(horizontalSpacer_14);


        vboxLayout3->addLayout(hboxLayout17);

        alwaysSnippetsCB = new QCheckBox(tab_reslist);
        alwaysSnippetsCB->setObjectName("alwaysSnippetsCB");
        alwaysSnippetsCB->setChecked(false);

        vboxLayout3->addWidget(alwaysSnippetsCB);

        tabptr1 = new QLabel(tab_reslist);
        tabptr1->setObjectName("tabptr1");
        tabptr1->setWordWrap(false);

        vboxLayout3->addWidget(tabptr1);

        line4 = new QFrame(tab_reslist);
        line4->setObjectName("line4");
        line4->setFrameShape(QFrame::Shape::HLine);
        line4->setFrameShadow(QFrame::Shadow::Sunken);

        vboxLayout3->addWidget(line4);

        hboxLayout18 = new QHBoxLayout();
        hboxLayout18->setSpacing(6);
        hboxLayout18->setObjectName("hboxLayout18");
        tLSTL1 = new QLabel(tab_reslist);
        tLSTL1->setObjectName("tLSTL1");
        tLSTL1->setWordWrap(false);

        hboxLayout18->addWidget(tLSTL1);

        snipCssPB = new QPushButton(tab_reslist);
        snipCssPB->setObjectName("snipCssPB");

        hboxLayout18->addWidget(snipCssPB);

        resetSnipCssPB = new QPushButton(tab_reslist);
        resetSnipCssPB->setObjectName("resetSnipCssPB");

        hboxLayout18->addWidget(resetSnipCssPB);

        horizontalSpacer_15 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout18->addItem(horizontalSpacer_15);


        vboxLayout3->addLayout(hboxLayout18);

        hboxLayout19 = new QHBoxLayout();
        hboxLayout19->setSpacing(6);
        hboxLayout19->setObjectName("hboxLayout19");
        snipwMaxLenLBL = new QLabel(tab_reslist);
        snipwMaxLenLBL->setObjectName("snipwMaxLenLBL");
        sizePolicy.setHeightForWidth(snipwMaxLenLBL->sizePolicy().hasHeightForWidth());
        snipwMaxLenLBL->setSizePolicy(sizePolicy);
        snipwMaxLenLBL->setWordWrap(false);

        hboxLayout19->addWidget(snipwMaxLenLBL);

        snipwMaxLenSB = new QSpinBox(tab_reslist);
        snipwMaxLenSB->setObjectName("snipwMaxLenSB");
        snipwMaxLenSB->setMinimum(1);
        snipwMaxLenSB->setMaximum(10000000);
        snipwMaxLenSB->setSingleStep(10);
        snipwMaxLenSB->setValue(1000);

        hboxLayout19->addWidget(snipwMaxLenSB);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout19->addItem(horizontalSpacer_3);

        hboxLayout19->setStretch(2, 1);

        vboxLayout3->addLayout(hboxLayout19);

        snipwByPageCB = new QCheckBox(tab_reslist);
        snipwByPageCB->setObjectName("snipwByPageCB");
        snipwByPageCB->setChecked(false);

        vboxLayout3->addWidget(snipwByPageCB);


        vboxLayout2->addLayout(vboxLayout3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout2->addItem(verticalSpacer);

        tabWidget->addTab(tab_reslist, QString());
        tab_restable = new QWidget();
        tab_restable->setObjectName("tab_restable");
        verticalLayout_6 = new QVBoxLayout(tab_restable);
        verticalLayout_6->setSpacing(6);
        verticalLayout_6->setContentsMargins(11, 11, 11, 11);
        verticalLayout_6->setObjectName("verticalLayout_6");
        noResTableHeaderCB = new QCheckBox(tab_restable);
        noResTableHeaderCB->setObjectName("noResTableHeaderCB");
        noResTableHeaderCB->setChecked(false);

        verticalLayout_6->addWidget(noResTableHeaderCB);

        showResTableVHeaderCB = new QCheckBox(tab_restable);
        showResTableVHeaderCB->setObjectName("showResTableVHeaderCB");
        showResTableVHeaderCB->setChecked(false);

        verticalLayout_6->addWidget(showResTableVHeaderCB);

        noRowJumpShortcutsCB = new QCheckBox(tab_restable);
        noRowJumpShortcutsCB->setObjectName("noRowJumpShortcutsCB");
        noRowJumpShortcutsCB->setChecked(false);

        verticalLayout_6->addWidget(noRowJumpShortcutsCB);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        restabShowTxtLBL = new QLabel(tab_restable);
        restabShowTxtLBL->setObjectName("restabShowTxtLBL");

        horizontalLayout->addWidget(restabShowTxtLBL);

        restabShowTxtNoShiftRB = new QRadioButton(tab_restable);
        restabShowTxtNoShiftRB->setObjectName("restabShowTxtNoShiftRB");

        horizontalLayout->addWidget(restabShowTxtNoShiftRB);

        restabShowTxtShiftRB = new QRadioButton(tab_restable);
        restabShowTxtShiftRB->setObjectName("restabShowTxtShiftRB");

        horizontalLayout->addWidget(restabShowTxtShiftRB);


        verticalLayout_6->addLayout(horizontalLayout);

        resTableNoHoverMetaCB = new QCheckBox(tab_restable);
        resTableNoHoverMetaCB->setObjectName("resTableNoHoverMetaCB");
        resTableNoHoverMetaCB->setChecked(false);

        verticalLayout_6->addWidget(resTableNoHoverMetaCB);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer_2);

        tabWidget->addTab(tab_restable, QString());
        tab_preview = new QWidget();
        tab_preview->setObjectName("tab_preview");
        verticalLayout_5 = new QVBoxLayout(tab_preview);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setContentsMargins(11, 11, 11, 11);
        verticalLayout_5->setObjectName("verticalLayout_5");
        hboxLayout20 = new QHBoxLayout();
        hboxLayout20->setSpacing(6);
        hboxLayout20->setObjectName("hboxLayout20");
        textLabelHL = new QLabel(tab_preview);
        textLabelHL->setObjectName("textLabelHL");
        sizePolicy.setHeightForWidth(textLabelHL->sizePolicy().hasHeightForWidth());
        textLabelHL->setSizePolicy(sizePolicy);
        textLabelHL->setWordWrap(false);

        hboxLayout20->addWidget(textLabelHL);

        maxHLTSB = new QSpinBox(tab_preview);
        maxHLTSB->setObjectName("maxHLTSB");
        maxHLTSB->setMinimum(0);
        maxHLTSB->setMaximum(100000);
        maxHLTSB->setSingleStep(500);
        maxHLTSB->setValue(3000);

        hboxLayout20->addWidget(maxHLTSB);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout20->addItem(horizontalSpacer_4);

        hboxLayout20->setStretch(2, 1);

        verticalLayout_5->addLayout(hboxLayout20);

        previewHtmlCB = new QCheckBox(tab_preview);
        previewHtmlCB->setObjectName("previewHtmlCB");
        previewHtmlCB->setChecked(false);

        verticalLayout_5->addWidget(previewHtmlCB);

        previewdarkbgCB = new QCheckBox(tab_preview);
        previewdarkbgCB->setObjectName("previewdarkbgCB");
        previewdarkbgCB->setChecked(false);

        verticalLayout_5->addWidget(previewdarkbgCB);

        previewActiveLinksCB = new QCheckBox(tab_preview);
        previewActiveLinksCB->setObjectName("previewActiveLinksCB");
        previewActiveLinksCB->setChecked(false);

        verticalLayout_5->addWidget(previewActiveLinksCB);

        hboxLayout21 = new QHBoxLayout();
        hboxLayout21->setSpacing(6);
        hboxLayout21->setObjectName("hboxLayout21");
        pvmaxfldlenLBL = new QLabel(tab_preview);
        pvmaxfldlenLBL->setObjectName("pvmaxfldlenLBL");
        sizePolicy.setHeightForWidth(pvmaxfldlenLBL->sizePolicy().hasHeightForWidth());
        pvmaxfldlenLBL->setSizePolicy(sizePolicy);
        pvmaxfldlenLBL->setWordWrap(false);

        hboxLayout21->addWidget(pvmaxfldlenLBL);

        pvmaxfldlenSB = new QSpinBox(tab_preview);
        pvmaxfldlenSB->setObjectName("pvmaxfldlenSB");
        pvmaxfldlenSB->setMinimum(0);
        pvmaxfldlenSB->setMaximum(10000);
        pvmaxfldlenSB->setValue(200);

        hboxLayout21->addWidget(pvmaxfldlenSB);

        pvmaxfldlenHSPC = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout21->addItem(pvmaxfldlenHSPC);

        hboxLayout21->setStretch(2, 1);

        verticalLayout_5->addLayout(hboxLayout21);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label = new QLabel(tab_preview);
        label->setObjectName("label");

        horizontalLayout_3->addWidget(label);

        plainBRRB = new QRadioButton(tab_preview);
        buttonGroup = new QButtonGroup(uiPrefsDialogBase);
        buttonGroup->setObjectName("buttonGroup");
        buttonGroup->addButton(plainBRRB);
        plainBRRB->setObjectName("plainBRRB");

        horizontalLayout_3->addWidget(plainBRRB);

        plainPRERB = new QRadioButton(tab_preview);
        buttonGroup->addButton(plainPRERB);
        plainPRERB->setObjectName("plainPRERB");

        horizontalLayout_3->addWidget(plainPRERB);

        plainPREWRAPRB = new QRadioButton(tab_preview);
        buttonGroup->addButton(plainPREWRAPRB);
        plainPREWRAPRB->setObjectName("plainPREWRAPRB");

        horizontalLayout_3->addWidget(plainPREWRAPRB);


        verticalLayout_5->addLayout(horizontalLayout_3);

        hboxLayout22 = new QHBoxLayout();
        hboxLayout22->setSpacing(6);
        hboxLayout22->setObjectName("hboxLayout22");
        previewLinesOverAnchorLBL = new QLabel(tab_preview);
        previewLinesOverAnchorLBL->setObjectName("previewLinesOverAnchorLBL");
        sizePolicy.setHeightForWidth(previewLinesOverAnchorLBL->sizePolicy().hasHeightForWidth());
        previewLinesOverAnchorLBL->setSizePolicy(sizePolicy);
        previewLinesOverAnchorLBL->setWordWrap(false);

        hboxLayout22->addWidget(previewLinesOverAnchorLBL);

        previewLinesOverAnchorSB = new QSpinBox(tab_preview);
        previewLinesOverAnchorSB->setObjectName("previewLinesOverAnchorSB");
        previewLinesOverAnchorSB->setMinimum(0);
        previewLinesOverAnchorSB->setMaximum(99);
        previewLinesOverAnchorSB->setValue(4);

        hboxLayout22->addWidget(previewLinesOverAnchorSB);

        previewLinesOverAnchorHSPC = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout22->addItem(previewLinesOverAnchorHSPC);

        hboxLayout22->setStretch(2, 1);

        verticalLayout_5->addLayout(hboxLayout22);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_3);

        tabWidget->addTab(tab_preview, QString());
        tab_shortcuts = new QWidget();
        tab_shortcuts->setObjectName("tab_shortcuts");
        verticalLayout_4 = new QVBoxLayout(tab_shortcuts);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        shortcutsTB = new QTableWidget(tab_shortcuts);
        shortcutsTB->setObjectName("shortcutsTB");

        verticalLayout->addWidget(shortcutsTB);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_2 = new QLabel(tab_shortcuts);
        label_2->setObjectName("label_2");

        horizontalLayout_2->addWidget(label_2);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        resetscPB = new QPushButton(tab_shortcuts);
        resetscPB->setObjectName("resetscPB");

        horizontalLayout_2->addWidget(resetscPB);


        verticalLayout->addLayout(horizontalLayout_2);


        verticalLayout_4->addLayout(verticalLayout);

        tabWidget->addTab(tab_shortcuts, QString());
        tab_search = new QWidget();
        tab_search->setObjectName("tab_search");
        vboxLayout4 = new QVBoxLayout(tab_search);
        vboxLayout4->setSpacing(6);
        vboxLayout4->setContentsMargins(11, 11, 11, 11);
        vboxLayout4->setObjectName("vboxLayout4");
        collapseDupsCB = new QCheckBox(tab_search);
        collapseDupsCB->setObjectName("collapseDupsCB");
        collapseDupsCB->setChecked(false);

        vboxLayout4->addWidget(collapseDupsCB);

        vboxLayout5 = new QVBoxLayout();
        vboxLayout5->setSpacing(6);
        vboxLayout5->setObjectName("vboxLayout5");
        hboxLayout23 = new QHBoxLayout();
        hboxLayout23->setSpacing(6);
        hboxLayout23->setObjectName("hboxLayout23");
        textLabel1_2 = new QLabel(tab_search);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        hboxLayout23->addWidget(textLabel1_2);

        stemLangCMB = new QComboBox(tab_search);
        stemLangCMB->setObjectName("stemLangCMB");

        hboxLayout23->addWidget(stemLangCMB);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout23->addItem(horizontalSpacer_11);

        hboxLayout23->setStretch(2, 1);

        vboxLayout5->addLayout(hboxLayout23);

        line5 = new QFrame(tab_search);
        line5->setObjectName("line5");
        line5->setFrameShape(QFrame::HLine);
        line5->setFrameShadow(QFrame::Sunken);

        vboxLayout5->addWidget(line5);

        autoPhraseCB = new QCheckBox(tab_search);
        autoPhraseCB->setObjectName("autoPhraseCB");

        vboxLayout5->addWidget(autoPhraseCB);

        hboxLayout24 = new QHBoxLayout();
        hboxLayout24->setSpacing(6);
        hboxLayout24->setObjectName("hboxLayout24");
        textLabel33 = new QLabel(tab_search);
        textLabel33->setObjectName("textLabel33");
        sizePolicy.setHeightForWidth(textLabel33->sizePolicy().hasHeightForWidth());
        textLabel33->setSizePolicy(sizePolicy);
        textLabel33->setWordWrap(false);

        hboxLayout24->addWidget(textLabel33);

        autoPThreshSB = new QDoubleSpinBox(tab_search);
        autoPThreshSB->setObjectName("autoPThreshSB");
        autoPThreshSB->setSingleStep(0.200000000000000);
        autoPThreshSB->setValue(2.000000000000000);

        hboxLayout24->addWidget(autoPThreshSB);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout24->addItem(horizontalSpacer_6);

        hboxLayout24->setStretch(2, 1);

        vboxLayout5->addLayout(hboxLayout24);

        line6 = new QFrame(tab_search);
        line6->setObjectName("line6");
        line6->setFrameShape(QFrame::HLine);
        line6->setFrameShadow(QFrame::Sunken);

        vboxLayout5->addWidget(line6);

        buildAbsCB = new QCheckBox(tab_search);
        buildAbsCB->setObjectName("buildAbsCB");

        vboxLayout5->addWidget(buildAbsCB);

        replAbsCB = new QCheckBox(tab_search);
        replAbsCB->setObjectName("replAbsCB");

        vboxLayout5->addWidget(replAbsCB);

        hboxLayout25 = new QHBoxLayout();
        hboxLayout25->setSpacing(6);
        hboxLayout25->setObjectName("hboxLayout25");
        textLabel2 = new QLabel(tab_search);
        textLabel2->setObjectName("textLabel2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(2);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textLabel2->sizePolicy().hasHeightForWidth());
        textLabel2->setSizePolicy(sizePolicy1);
        textLabel2->setWordWrap(false);

        hboxLayout25->addWidget(textLabel2);

        syntlenSB = new QSpinBox(tab_search);
        syntlenSB->setObjectName("syntlenSB");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(syntlenSB->sizePolicy().hasHeightForWidth());
        syntlenSB->setSizePolicy(sizePolicy2);
        syntlenSB->setMinimum(80);
        syntlenSB->setMaximum(100000);
        syntlenSB->setSingleStep(10);
        syntlenSB->setValue(250);

        hboxLayout25->addWidget(syntlenSB);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout25->addItem(horizontalSpacer_7);

        hboxLayout25->setStretch(2, 1);

        vboxLayout5->addLayout(hboxLayout25);

        hboxLayout26 = new QHBoxLayout();
        hboxLayout26->setSpacing(6);
        hboxLayout26->setObjectName("hboxLayout26");
        textLabel3 = new QLabel(tab_search);
        textLabel3->setObjectName("textLabel3");
        sizePolicy.setHeightForWidth(textLabel3->sizePolicy().hasHeightForWidth());
        textLabel3->setSizePolicy(sizePolicy);
        textLabel3->setWordWrap(false);

        hboxLayout26->addWidget(textLabel3);

        syntctxSB = new QSpinBox(tab_search);
        syntctxSB->setObjectName("syntctxSB");
        syntctxSB->setMinimum(2);
        syntctxSB->setMaximum(20000);
        syntctxSB->setValue(4);

        hboxLayout26->addWidget(syntctxSB);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout26->addItem(horizontalSpacer_8);

        hboxLayout26->setStretch(2, 1);

        vboxLayout5->addLayout(hboxLayout26);

        line7 = new QFrame(tab_search);
        line7->setObjectName("line7");
        line7->setFrameShape(QFrame::HLine);
        line7->setFrameShadow(QFrame::Sunken);

        vboxLayout5->addWidget(line7);

        hboxLayout27 = new QHBoxLayout();
        hboxLayout27->setSpacing(6);
        hboxLayout27->setObjectName("hboxLayout27");
        textLabel14 = new QLabel(tab_search);
        textLabel14->setObjectName("textLabel14");
        sizePolicy.setHeightForWidth(textLabel14->sizePolicy().hasHeightForWidth());
        textLabel14->setSizePolicy(sizePolicy);
        textLabel14->setWordWrap(false);

        hboxLayout27->addWidget(textLabel14);

        autoSuffsCB = new QCheckBox(tab_search);
        autoSuffsCB->setObjectName("autoSuffsCB");

        hboxLayout27->addWidget(autoSuffsCB);

        autoSuffsLE = new QLineEdit(tab_search);
        autoSuffsLE->setObjectName("autoSuffsLE");
        autoSuffsLE->setMinimumSize(QSize(30, 0));

        hboxLayout27->addWidget(autoSuffsLE);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout27->addItem(horizontalSpacer_9);

        hboxLayout27->setStretch(3, 1);

        vboxLayout5->addLayout(hboxLayout27);

        hboxLayout28 = new QHBoxLayout();
        hboxLayout28->setSpacing(6);
        hboxLayout28->setObjectName("hboxLayout28");
        autoSpellCB = new QCheckBox(tab_search);
        autoSpellCB->setObjectName("autoSpellCB");

        hboxLayout28->addWidget(autoSpellCB);

        autoSpellLBL = new QLabel(tab_search);
        autoSpellLBL->setObjectName("autoSpellLBL");

        hboxLayout28->addWidget(autoSpellLBL);

        autoSpellMaxDistSB = new QSpinBox(tab_search);
        autoSpellMaxDistSB->setObjectName("autoSpellMaxDistSB");
        autoSpellMaxDistSB->setMinimum(0);
        autoSpellMaxDistSB->setMaximum(100);
        autoSpellMaxDistSB->setValue(1);

        hboxLayout28->addWidget(autoSpellMaxDistSB);

        horizontalSpacer_91 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout28->addItem(horizontalSpacer_91);

        hboxLayout28->setStretch(3, 1);

        vboxLayout5->addLayout(hboxLayout28);

        hboxLayout29 = new QHBoxLayout();
        hboxLayout29->setSpacing(6);
        hboxLayout29->setObjectName("hboxLayout29");
        textLabelSG = new QLabel(tab_search);
        textLabelSG->setObjectName("textLabelSG");
        sizePolicy.setHeightForWidth(textLabelSG->sizePolicy().hasHeightForWidth());
        textLabelSG->setSizePolicy(sizePolicy);
        textLabelSG->setWordWrap(false);

        hboxLayout29->addWidget(textLabelSG);

        synFileCB = new QCheckBox(tab_search);
        synFileCB->setObjectName("synFileCB");

        hboxLayout29->addWidget(synFileCB);

        synFilePB = new QPushButton(tab_search);
        synFilePB->setObjectName("synFilePB");
        synFilePB->setMinimumSize(QSize(30, 0));

        hboxLayout29->addWidget(synFilePB);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout29->addItem(horizontalSpacer_10);

        hboxLayout29->setStretch(3, 1);

        vboxLayout5->addLayout(hboxLayout29);

        ignwildsCB = new QCheckBox(tab_search);
        ignwildsCB->setObjectName("ignwildsCB");
        ignwildsCB->setChecked(false);

        vboxLayout5->addWidget(ignwildsCB);

        spacer2 = new QSpacerItem(20, 70, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout5->addItem(spacer2);


        vboxLayout4->addLayout(vboxLayout5);

        tabWidget->addTab(tab_search, QString());
        tab_extradbs = new QWidget();
        tab_extradbs->setObjectName("tab_extradbs");
        vboxLayout6 = new QVBoxLayout(tab_extradbs);
        vboxLayout6->setSpacing(6);
        vboxLayout6->setContentsMargins(11, 11, 11, 11);
        vboxLayout6->setObjectName("vboxLayout6");
        vboxLayout7 = new QVBoxLayout();
        vboxLayout7->setSpacing(6);
        vboxLayout7->setObjectName("vboxLayout7");
        idxLV = new QListWidget(tab_extradbs);
        idxLV->setObjectName("idxLV");
        idxLV->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout7->addWidget(idxLV);

        hboxLayout30 = new QHBoxLayout();
        hboxLayout30->setSpacing(6);
        hboxLayout30->setObjectName("hboxLayout30");
        togExtraDbPB = new QPushButton(tab_extradbs);
        togExtraDbPB->setObjectName("togExtraDbPB");

        hboxLayout30->addWidget(togExtraDbPB);

        actAllExtraDbPB = new QPushButton(tab_extradbs);
        actAllExtraDbPB->setObjectName("actAllExtraDbPB");

        hboxLayout30->addWidget(actAllExtraDbPB);

        unacAllExtraDbPB = new QPushButton(tab_extradbs);
        unacAllExtraDbPB->setObjectName("unacAllExtraDbPB");

        hboxLayout30->addWidget(unacAllExtraDbPB);

        ptransPB = new QPushButton(tab_extradbs);
        ptransPB->setObjectName("ptransPB");

        hboxLayout30->addWidget(ptransPB);


        vboxLayout7->addLayout(hboxLayout30);

        line9 = new QFrame(tab_extradbs);
        line9->setObjectName("line9");
        line9->setFrameShape(QFrame::HLine);
        line9->setFrameShadow(QFrame::Sunken);

        vboxLayout7->addWidget(line9);

        hboxLayout31 = new QHBoxLayout();
        hboxLayout31->setSpacing(6);
        hboxLayout31->setObjectName("hboxLayout31");
        delExtraDbPB = new QPushButton(tab_extradbs);
        delExtraDbPB->setObjectName("delExtraDbPB");

        hboxLayout31->addWidget(delExtraDbPB);

        spacer3 = new QSpacerItem(16, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout31->addItem(spacer3);

        addExtraDbPB = new QPushButton(tab_extradbs);
        addExtraDbPB->setObjectName("addExtraDbPB");
        addExtraDbPB->setEnabled(true);

        hboxLayout31->addWidget(addExtraDbPB);


        vboxLayout7->addLayout(hboxLayout31);


        vboxLayout6->addLayout(vboxLayout7);

        tabWidget->addTab(tab_extradbs, QString());
        tab_misc = new QWidget();
        tab_misc->setObjectName("tab_misc");
        verticalLayout_2 = new QVBoxLayout(tab_misc);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        anchorTamilHackCB = new QCheckBox(tab_misc);
        anchorTamilHackCB->setObjectName("anchorTamilHackCB");

        verticalLayout_2->addWidget(anchorTamilHackCB);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_4);

        tabWidget->addTab(tab_misc, QString());

        vboxLayout->addWidget(tabWidget);

        hboxLayout32 = new QHBoxLayout();
        hboxLayout32->setSpacing(6);
        hboxLayout32->setObjectName("hboxLayout32");
        Horizontal_Spacing2 = new QSpacerItem(210, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout32->addItem(Horizontal_Spacing2);

        buttonOk = new QPushButton(uiPrefsDialogBase);
        buttonOk->setObjectName("buttonOk");
        buttonOk->setAutoDefault(true);

        hboxLayout32->addWidget(buttonOk);

        buttonCancel = new QPushButton(uiPrefsDialogBase);
        buttonCancel->setObjectName("buttonCancel");
        buttonCancel->setAutoDefault(true);

        hboxLayout32->addWidget(buttonCancel);


        vboxLayout->addLayout(hboxLayout32);


        retranslateUi(uiPrefsDialogBase);

        tabWidget->setCurrentIndex(0);
        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(uiPrefsDialogBase);
    } // setupUi

    void retranslateUi(QDialog *uiPrefsDialogBase)
    {
        uiPrefsDialogBase->setWindowTitle(QCoreApplication::translate("uiPrefsDialogBase", "Recoll - User Preferences", nullptr));
        viewActionPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Choose editor applications", nullptr));
#if QT_CONFIG(tooltip)
        singleappCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "If set, starting a new instance on the same index will raise an existing one.", nullptr));
#endif // QT_CONFIG(tooltip)
        singleappCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Single application", nullptr));
        textLabelSST->setText(QCoreApplication::translate("uiPrefsDialogBase", "Start with simple search mode: ", nullptr));
#if QT_CONFIG(tooltip)
        hsLBL->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Limit the size of the search history. Use 0 to disable, -1 for unlimited.", nullptr));
#endif // QT_CONFIG(tooltip)
        hsLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Maximum size of search history (0: disable, -1: unlimited):", nullptr));
#if QT_CONFIG(tooltip)
        startLBL->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Things we open when starting up.", nullptr));
#endif // QT_CONFIG(tooltip)
        startLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Open at start:", nullptr));
        initStartAdvCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Advanced Search dialog", nullptr));
        startWithFragsToolOpenCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Query Fragments window", nullptr));
        keepSortCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Remember sort activation state.", nullptr));
#if QT_CONFIG(tooltip)
        textLabel20->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Set to 0 to disable and speed up startup by avoiding tree computation.", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel20->setText(QCoreApplication::translate("uiPrefsDialogBase", "Depth of side filter directory tree", nullptr));
#if QT_CONFIG(tooltip)
        sidefilterdateformatLBL->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "See Qt QDateTimeEdit documentation. E.g. yyyy-MM-dd. Leave empty to use the default Qt/System format.", nullptr));
#endif // QT_CONFIG(tooltip)
        sidefilterdateformatLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Side filter dates format (change needs restart)", nullptr));
#if QT_CONFIG(tooltip)
        label1->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Decide if document filters are shown as radio buttons, toolbar combobox, or menu.", nullptr));
#endif // QT_CONFIG(tooltip)
        label1->setText(QCoreApplication::translate("uiPrefsDialogBase", "Document category choice style:", nullptr));
        filterBT_RB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Buttons Panel", nullptr));
        filterCMB_RB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Toolbar Combobox", nullptr));
        filterMN_RB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Menu", nullptr));
#if QT_CONFIG(tooltip)
        labeltb->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Hide some user interface elements.", nullptr));
#endif // QT_CONFIG(tooltip)
        labeltb->setText(QCoreApplication::translate("uiPrefsDialogBase", "Hide:", nullptr));
        noToolbarsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Toolbars", nullptr));
        noStatusBarCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Status bar", nullptr));
#if QT_CONFIG(tooltip)
        noMenuBarCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Show button instead.", nullptr));
#endif // QT_CONFIG(tooltip)
        noMenuBarCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Menu bar", nullptr));
#if QT_CONFIG(tooltip)
        noSSTypCMBCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Show choice in menu only.", nullptr));
#endif // QT_CONFIG(tooltip)
        noSSTypCMBCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Search type", nullptr));
        noClearSearchCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Clear/Search", nullptr));
#if QT_CONFIG(tooltip)
        noSizeFilterCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Do not show the File Size tool in the side filters section", nullptr));
#endif // QT_CONFIG(tooltip)
        noSizeFilterCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "File Size side filter", nullptr));
        showTrayIconCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Show system tray icon.", nullptr));
        closeToTrayCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Close to tray instead of exiting.", nullptr));
        trayMessagesCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Generate desktop notifications.", nullptr));
        noBeepsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Suppress all beeps.", nullptr));
        showTempFileWarningCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Show warning when opening temporary file.", nullptr));
        ssNoCompleteCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Disable Qt autocompletion in search entry.", nullptr));
#if QT_CONFIG(tooltip)
        ssearchCompletePassiveCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "The completion only changes the entry when activated.", nullptr));
#endif // QT_CONFIG(tooltip)
        ssearchCompletePassiveCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Completion: no automatic line editing.", nullptr));
        ssSearchOnCompleteCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Start search on completer popup activation.", nullptr));
#if QT_CONFIG(tooltip)
        ssearchCompleterHistCntLBL->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Maximum number of history entries in completer list", nullptr));
#endif // QT_CONFIG(tooltip)
        ssearchCompleterHistCntLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Number of history entries in completer:", nullptr));
#if QT_CONFIG(tooltip)
        showcompleterhitcountsCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Displays the total number of occurences of the term in the index", nullptr));
#endif // QT_CONFIG(tooltip)
        showcompleterhitcountsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Show hit counts in completer popup.", nullptr));
        textLabelqtc->setText(QCoreApplication::translate("uiPrefsDialogBase", "Highlight CSS style for query terms", nullptr));
#if QT_CONFIG(tooltip)
        qtermStyleCMB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Query terms highlighting in results. <br>Maybe try something like \"color:red;background:yellow\" for something more lively than the default blue...", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        wuiscaleLBL->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Zoom factor for the user interface. Useful if the default is not right for your screen resolution.", nullptr));
#endif // QT_CONFIG(tooltip)
        wuiscaleLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Display scale (default 1.0):", nullptr));
        colorschemeLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Color scheme", nullptr));
        tLSTL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Application Qt style sheet", nullptr));
#if QT_CONFIG(tooltip)
        resetSSPB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Resets the style sheet to default", nullptr));
#endif // QT_CONFIG(tooltip)
        resetSSPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "None (default)", nullptr));
#if QT_CONFIG(tooltip)
        stylesheetPB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Opens a dialog to select the style sheet file.<br>Look at /usr/share/recoll/examples/recoll[-dark].qss for an example.", nullptr));
#endif // QT_CONFIG(tooltip)
        stylesheetPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Choose QSS File", nullptr));
#if QT_CONFIG(tooltip)
        uilanguageLBL->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Note: most translations are incomplete. Leave empty to use the system environment.", nullptr));
#endif // QT_CONFIG(tooltip)
        uilanguageLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Interface language (needs restart):", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_ui), QCoreApplication::translate("uiPrefsDialogBase", "User interface", nullptr));
        textLabel1->setText(QCoreApplication::translate("uiPrefsDialogBase", "Number of entries in a result page", nullptr));
        textLabel4->setText(QCoreApplication::translate("uiPrefsDialogBase", "Result list font", nullptr));
#if QT_CONFIG(tooltip)
        reslistFontPB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Opens a dialog to select the result list font", nullptr));
#endif // QT_CONFIG(tooltip)
        reslistFontPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Helvetica-10", nullptr));
#if QT_CONFIG(tooltip)
        resetFontPB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Resets the result list font to the system default", nullptr));
#endif // QT_CONFIG(tooltip)
        resetFontPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Reset", nullptr));
        CLEditPara->setText(QCoreApplication::translate("uiPrefsDialogBase", "Edit result paragraph format string", nullptr));
        CLEditHeader->setText(QCoreApplication::translate("uiPrefsDialogBase", "Edit result page html header insert", nullptr));
        textLabelDateFormat->setText(QCoreApplication::translate("uiPrefsDialogBase", "Date format (strftime(3))", nullptr));
        textLabelAbss->setText(QCoreApplication::translate("uiPrefsDialogBase", "Abstract snippet separator", nullptr));
        alwaysSnippetsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Display a Snippets link even if the document has no pages (needs restart).", nullptr));
        tabptr1->setText(QCoreApplication::translate("uiPrefsDialogBase", "<em>Also see the \"synthetic abstract\" parameters in the Search parameters tab", nullptr));
#if QT_CONFIG(tooltip)
        tLSTL1->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "User style to apply to the snippets window.<br> Note: the result page header insert is also included in the snippets window header.", nullptr));
#endif // QT_CONFIG(tooltip)
        tLSTL1->setText(QCoreApplication::translate("uiPrefsDialogBase", "Snippets window CSS file", nullptr));
#if QT_CONFIG(tooltip)
        snipCssPB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Opens a dialog to select the Snippets window CSS style sheet file", nullptr));
#endif // QT_CONFIG(tooltip)
        snipCssPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Choose", nullptr));
#if QT_CONFIG(tooltip)
        resetSnipCssPB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Resets the Snippets window style", nullptr));
#endif // QT_CONFIG(tooltip)
        resetSnipCssPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Reset", nullptr));
        snipwMaxLenLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Maximum number of snippets displayed in the snippets window", nullptr));
        snipwByPageCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Sort snippets by page number (default: by weight).", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_reslist), QCoreApplication::translate("uiPrefsDialogBase", "Result List", nullptr));
        noResTableHeaderCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Hide result table header.", nullptr));
        showResTableVHeaderCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Show result table row headers.", nullptr));
        noRowJumpShortcutsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Disable the Ctrl+[0-9]/Shift+[a-z] shortcuts for jumping to table rows.", nullptr));
        restabShowTxtLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "To display document text instead of metadata in result table detail area, use:", nullptr));
        restabShowTxtNoShiftRB->setText(QCoreApplication::translate("uiPrefsDialogBase", "left mouse click", nullptr));
        restabShowTxtShiftRB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Shift+click", nullptr));
        resTableNoHoverMetaCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Do not display metadata when hovering over rows.", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_restable), QCoreApplication::translate("uiPrefsDialogBase", "Result Table", nullptr));
#if QT_CONFIG(tooltip)
        textLabelHL->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Texts over this size will not be highlighted in preview (too slow).", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabelHL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Maximum text size highlighted for preview (kilobytes)", nullptr));
        previewHtmlCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Prefer HTML to plain text for preview.", nullptr));
#if QT_CONFIG(tooltip)
        previewdarkbgCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "When displaying HTML in dark mode, try to use a dark background (and light color).<br>This conflicts with many documents which will override our light setting with their own CSS (dark) foreground color, resulting in an unreadable document, so it is off by default.", nullptr));
#endif // QT_CONFIG(tooltip)
        previewdarkbgCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Use dark background when displaying HTML in dark mode.", nullptr));
#if QT_CONFIG(tooltip)
        previewActiveLinksCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Make links inside the preview window clickable, and start an external browser when they are clicked.", nullptr));
#endif // QT_CONFIG(tooltip)
        previewActiveLinksCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Activate links in preview.", nullptr));
#if QT_CONFIG(tooltip)
        pvmaxfldlenLBL->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Set to 0 to disable details/summary feature", nullptr));
#endif // QT_CONFIG(tooltip)
        pvmaxfldlenLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Fields display: max field length before using summary:", nullptr));
#if QT_CONFIG(tooltip)
        label->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Lines in PRE text are not folded. Using BR loses some indentation. PRE + Wrap style may be what you want.", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("uiPrefsDialogBase", "Plain text to HTML line style", nullptr));
        plainBRRB->setText(QCoreApplication::translate("uiPrefsDialogBase", "<BR>", nullptr));
        plainPRERB->setText(QCoreApplication::translate("uiPrefsDialogBase", "<PRE>", nullptr));
        plainPREWRAPRB->setText(QCoreApplication::translate("uiPrefsDialogBase", "<PRE> + wrap", nullptr));
#if QT_CONFIG(tooltip)
        previewLinesOverAnchorLBL->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Number of lines to be shown over a search term found by preview search.", nullptr));
#endif // QT_CONFIG(tooltip)
        previewLinesOverAnchorLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Search term line offset:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_preview), QCoreApplication::translate("uiPrefsDialogBase", "Preview", nullptr));
        label_2->setText(QCoreApplication::translate("uiPrefsDialogBase", "Use F1 to  access the manual", nullptr));
        resetscPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Reset shortcuts defaults", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_shortcuts), QCoreApplication::translate("uiPrefsDialogBase", "Shortcuts", nullptr));
#if QT_CONFIG(tooltip)
        collapseDupsCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "If checked, results with the same content under different names will only be shown once.", nullptr));
#endif // QT_CONFIG(tooltip)
        collapseDupsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Hide duplicate results.", nullptr));
        textLabel1_2->setText(QCoreApplication::translate("uiPrefsDialogBase", "Stemming language", nullptr));
#if QT_CONFIG(tooltip)
        autoPhraseCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "A search for [rolling stones] (2 terms) will be changed to [rolling or stones or (rolling phrase 2 stones)]. \n"
"This should give higher precedence to the results where the search terms appear exactly as entered.", nullptr));
#endif // QT_CONFIG(tooltip)
        autoPhraseCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Automatically add phrase to simple searches", nullptr));
#if QT_CONFIG(tooltip)
        textLabel33->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Frequency percentage threshold over which we do not use terms inside autophrase. \n"
"Frequent terms are a major performance issue with phrases. \n"
"Skipped terms augment the phrase slack, and reduce the autophrase efficiency.\n"
"The default value is 2 (percent). ", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel33->setText(QCoreApplication::translate("uiPrefsDialogBase", "Autophrase term frequency threshold percentage", nullptr));
#if QT_CONFIG(tooltip)
        buildAbsCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Do we try to build abstracts for result list entries by using the context of query terms ? \n"
"May be slow for big documents.", nullptr));
#endif // QT_CONFIG(tooltip)
        buildAbsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Dynamically build abstracts", nullptr));
#if QT_CONFIG(tooltip)
        replAbsCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Do we synthetize an abstract even if the document seemed to have one?", nullptr));
#endif // QT_CONFIG(tooltip)
        replAbsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Replace abstracts from documents", nullptr));
        textLabel2->setText(QCoreApplication::translate("uiPrefsDialogBase", "Synthetic abstract size (characters)", nullptr));
        textLabel3->setText(QCoreApplication::translate("uiPrefsDialogBase", "Synthetic abstract context words", nullptr));
#if QT_CONFIG(tooltip)
        textLabel14->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "The words in the list will be automatically turned to ext:xxx clauses in the query language entry.", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel14->setText(QCoreApplication::translate("uiPrefsDialogBase", "Query language magic file name suffixes.", nullptr));
        autoSuffsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Enable", nullptr));
#if QT_CONFIG(tooltip)
        autoSpellCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Add common spelling approximations for rare terms.", nullptr));
#endif // QT_CONFIG(tooltip)
        autoSpellCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Automatic spelling approximation.", nullptr));
        autoSpellLBL->setText(QCoreApplication::translate("uiPrefsDialogBase", "Max spelling distance", nullptr));
        textLabelSG->setText(QCoreApplication::translate("uiPrefsDialogBase", "Synonyms file", nullptr));
        synFileCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Enable", nullptr));
        synFilePB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Choose", nullptr));
#if QT_CONFIG(tooltip)
        ignwildsCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Wild card characters *?[] will processed as punctuation instead of being expanded", nullptr));
#endif // QT_CONFIG(tooltip)
        ignwildsCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Ignore wild card characters in ALL terms and ANY terms modes", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_search), QCoreApplication::translate("uiPrefsDialogBase", "Search parameters", nullptr));
        togExtraDbPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Toggle selected", nullptr));
        actAllExtraDbPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Activate All", nullptr));
        unacAllExtraDbPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Deactivate All", nullptr));
#if QT_CONFIG(tooltip)
        ptransPB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Set path translations for the selected index or for the main one if no selection exists.", nullptr));
#endif // QT_CONFIG(tooltip)
        ptransPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Paths translations", nullptr));
#if QT_CONFIG(tooltip)
        delExtraDbPB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Remove from list. This has no effect on the disk index.", nullptr));
#endif // QT_CONFIG(tooltip)
        delExtraDbPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Remove selected", nullptr));
#if QT_CONFIG(tooltip)
        addExtraDbPB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Click to add another index directory to the list. You can select either a Recoll configuration directory or a Xapian index.", nullptr));
#endif // QT_CONFIG(tooltip)
        addExtraDbPB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Add index", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_extradbs), QCoreApplication::translate("uiPrefsDialogBase", "External Indexes", nullptr));
#if QT_CONFIG(tooltip)
        anchorTamilHackCB->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "The bug causes a strange circle characters to be displayed inside highlighted Tamil words. The workaround inserts an additional space character which appears to fix the problem.", nullptr));
#endif // QT_CONFIG(tooltip)
        anchorTamilHackCB->setText(QCoreApplication::translate("uiPrefsDialogBase", "Work around Tamil QTBUG-78923 by inserting space before anchor text", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_misc), QCoreApplication::translate("uiPrefsDialogBase", "Misc", nullptr));
#if QT_CONFIG(tooltip)
        buttonOk->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Apply changes", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonOk->setText(QCoreApplication::translate("uiPrefsDialogBase", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        buttonCancel->setToolTip(QCoreApplication::translate("uiPrefsDialogBase", "Discard changes", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCancel->setText(QCoreApplication::translate("uiPrefsDialogBase", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class uiPrefsDialogBase: public Ui_uiPrefsDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIPREFS_H
