/********************************************************************************
** Form generated from reading UI file 'crontool.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CRONTOOL_H
#define UI_CRONTOOL_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_CronToolW
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QGridLayout *gridLayout;
    QLabel *label_4;
    QLineEdit *daysLE;
    QLabel *label_3;
    QLineEdit *hoursLE;
    QLabel *label_2;
    QLineEdit *minsLE;
    QLabel *label_5;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *CronToolW)
    {
        if (CronToolW->objectName().isEmpty())
            CronToolW->setObjectName("CronToolW");
        CronToolW->resize(508, 416);
        verticalLayout_2 = new QVBoxLayout(CronToolW);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(CronToolW);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label_4 = new QLabel(CronToolW);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_4, 0, 0, 1, 1);

        daysLE = new QLineEdit(CronToolW);
        daysLE->setObjectName("daysLE");

        gridLayout->addWidget(daysLE, 0, 1, 1, 1);

        label_3 = new QLabel(CronToolW);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 1, 0, 1, 1);

        hoursLE = new QLineEdit(CronToolW);
        hoursLE->setObjectName("hoursLE");

        gridLayout->addWidget(hoursLE, 1, 1, 1, 1);

        label_2 = new QLabel(CronToolW);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        minsLE = new QLineEdit(CronToolW);
        minsLE->setObjectName("minsLE");

        gridLayout->addWidget(minsLE, 2, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        label_5 = new QLabel(CronToolW);
        label_5->setObjectName("label_5");
        label_5->setWordWrap(true);

        verticalLayout->addWidget(label_5);


        verticalLayout_2->addLayout(verticalLayout);

        buttonBox = new QDialogButtonBox(CronToolW);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(CronToolW);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, CronToolW, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, CronToolW, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(CronToolW);
    } // setupUi

    void retranslateUi(QDialog *CronToolW)
    {
        CronToolW->setWindowTitle(QCoreApplication::translate("CronToolW", "Cron Dialog", nullptr));
        label->setText(QCoreApplication::translate("CronToolW", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Recoll</span> batch indexing schedule (cron) </p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Each field can contain a wildcard (*), a single numeric value, comma-separated lists (1,3,5) and ranges (1-7). More generally, the fields will be used <span style=\" font-style:italic;\">as is</span> in"
                        "side the crontab file, and the full crontab syntax can be used, see crontab(5).</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br />For example, entering <span style=\" font-family:'Courier New,courier';\">*</span> in <span style=\" font-style:italic;\">Days, </span><span style=\" font-family:'Courier New,courier';\">12,19</span> in <span style=\" font-style:italic;\">Hours</span> and <span style=\" font-family:'Courier New,courier';\">15</span> in <span style=\" font-style:italic;\">Minutes</span> would start recollindex every day at 12:15 AM and 7:15 PM</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">A schedule with very frequent activations is probably less efficient than real time indexing.</p"
                        "></body></html>", nullptr));
        label_4->setText(QCoreApplication::translate("CronToolW", "Days of week (* or 0-7, 0 or 7 is Sunday)", nullptr));
        label_3->setText(QCoreApplication::translate("CronToolW", "Hours (* or 0-23)", nullptr));
        label_2->setText(QCoreApplication::translate("CronToolW", "Minutes (0-59)", nullptr));
        label_5->setText(QCoreApplication::translate("CronToolW", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Click <span style=\" font-style:italic;\">Disable</span> to stop automatic batch indexing, <span style=\" font-style:italic;\">Enable</span> to activate it, <span style=\" font-style:italic;\">Cancel</span> to change nothing.</p></body></html>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CronToolW: public Ui_CronToolW {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CRONTOOL_H
