/********************************************************************************
** Form generated from reading UI file 'ssearchb.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SSEARCHB_H
#define UI_SSEARCHB_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SSearchBase
{
public:
    QHBoxLayout *hboxLayout;
    QHBoxLayout *hboxLayout1;
    QPushButton *clearqPB;
    QPushButton *searchPB;
    QComboBox *searchTypCMB;
    QLineEdit *queryText;
    QPushButton *historyPB;
    QPushButton *menuPB;

    void setupUi(QWidget *SSearchBase)
    {
        if (SSearchBase->objectName().isEmpty())
            SSearchBase->setObjectName("SSearchBase");
        SSearchBase->resize(593, 48);
        hboxLayout = new QHBoxLayout(SSearchBase);
        hboxLayout->setSpacing(2);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(4, 4, 4, 4);
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(3);
        hboxLayout1->setObjectName("hboxLayout1");
        clearqPB = new QPushButton(SSearchBase);
        clearqPB->setObjectName("clearqPB");
        clearqPB->setEnabled(false);

        hboxLayout1->addWidget(clearqPB);

        searchPB = new QPushButton(SSearchBase);
        searchPB->setObjectName("searchPB");
        searchPB->setEnabled(false);

        hboxLayout1->addWidget(searchPB);

        searchTypCMB = new QComboBox(SSearchBase);
        searchTypCMB->setObjectName("searchTypCMB");
        searchTypCMB->setFocusPolicy(Qt::TabFocus);

        hboxLayout1->addWidget(searchTypCMB);

        queryText = new QLineEdit(SSearchBase);
        queryText->setObjectName("queryText");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(8);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(queryText->sizePolicy().hasHeightForWidth());
        queryText->setSizePolicy(sizePolicy);
        queryText->setMinimumSize(QSize(155, 0));

        hboxLayout1->addWidget(queryText);

        historyPB = new QPushButton(SSearchBase);
        historyPB->setObjectName("historyPB");
        historyPB->setEnabled(true);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/clock.png"), QSize(), QIcon::Mode::Normal, QIcon::State::On);
        historyPB->setIcon(icon);

        hboxLayout1->addWidget(historyPB);

        menuPB = new QPushButton(SSearchBase);
        menuPB->setObjectName("menuPB");
        menuPB->setEnabled(true);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/menu.png"), QSize(), QIcon::Mode::Normal, QIcon::State::On);
        menuPB->setIcon(icon1);

        hboxLayout1->addWidget(menuPB);


        hboxLayout->addLayout(hboxLayout1);


        retranslateUi(SSearchBase);

        QMetaObject::connectSlotsByName(SSearchBase);
    } // setupUi

    void retranslateUi(QWidget *SSearchBase)
    {
        SSearchBase->setWindowTitle(QCoreApplication::translate("SSearchBase", "SSearchBase", nullptr));
#if QT_CONFIG(tooltip)
        clearqPB->setToolTip(QCoreApplication::translate("SSearchBase", "Erase search entry", nullptr));
#endif // QT_CONFIG(tooltip)
        clearqPB->setText(QCoreApplication::translate("SSearchBase", "Clear", nullptr));
#if QT_CONFIG(shortcut)
        clearqPB->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        searchPB->setToolTip(QCoreApplication::translate("SSearchBase", "Start query", nullptr));
#endif // QT_CONFIG(tooltip)
        searchPB->setText(QCoreApplication::translate("SSearchBase", "Search", nullptr));
#if QT_CONFIG(shortcut)
        searchPB->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        searchTypCMB->setToolTip(QCoreApplication::translate("SSearchBase", "Choose search type.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        historyPB->setToolTip(QCoreApplication::translate("SSearchBase", "Show query history", nullptr));
#endif // QT_CONFIG(tooltip)
        historyPB->setText(QString());
#if QT_CONFIG(tooltip)
        menuPB->setToolTip(QCoreApplication::translate("SSearchBase", "Main menu", nullptr));
#endif // QT_CONFIG(tooltip)
        menuPB->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class SSearchBase: public Ui_SSearchBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SSEARCHB_H
