/********************************************************************************
** Form generated from reading UI file 'rtitool.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RTITOOL_H
#define UI_RTITOOL_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_RTIToolW
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QCheckBox *sesCB;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QCheckBox *nowCB;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *RTIToolW)
    {
        if (RTIToolW->objectName().isEmpty())
            RTIToolW->setObjectName("RTIToolW");
        RTIToolW->resize(423, 207);
        verticalLayout_2 = new QVBoxLayout(RTIToolW);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(RTIToolW);
        label->setObjectName("label");
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        sesCB = new QCheckBox(RTIToolW);
        sesCB->setObjectName("sesCB");

        verticalLayout->addWidget(sesCB);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(28, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        nowCB = new QCheckBox(RTIToolW);
        nowCB->setObjectName("nowCB");
        nowCB->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(nowCB->sizePolicy().hasHeightForWidth());
        nowCB->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(nowCB);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_2->addLayout(verticalLayout);

        buttonBox = new QDialogButtonBox(RTIToolW);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(RTIToolW);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, RTIToolW, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, RTIToolW, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(RTIToolW);
    } // setupUi

    void retranslateUi(QDialog *RTIToolW)
    {
        RTIToolW->setWindowTitle(QCoreApplication::translate("RTIToolW", "Real time indexing automatic start", nullptr));
        label->setText(QCoreApplication::translate("RTIToolW", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Recoll</span> indexing can be set up to run as a daemon, updating the index as files change, in real time. You gain an always up to date index, but system resources are used permanently.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p></body></html>", nullptr));
        sesCB->setText(QCoreApplication::translate("RTIToolW", "Start indexing daemon with my desktop session.", nullptr));
        nowCB->setText(QCoreApplication::translate("RTIToolW", "Also start indexing daemon right now.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RTIToolW: public Ui_RTIToolW {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RTITOOL_H
