/****************************************************************************
** Meta object code from reading C++ file 'confgui.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../qtgui/confgui/confgui.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'confgui.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7confgui9ConfTabsWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfTabsW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui9ConfTabsWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfTabsW",
        "sig_prefsChanged",
        "",
        "acceptChanges",
        "rejectChanges",
        "buttonClicked",
        "QAbstractButton*",
        "reloadPanels",
        "setCurrentIndex"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sig_prefsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'acceptChanges'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'rejectChanges'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'buttonClicked'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'reloadPanels'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCurrentIndex'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfTabsW, qt_meta_tag_ZN7confgui9ConfTabsWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfTabsW::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui9ConfTabsWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui9ConfTabsWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui9ConfTabsWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfTabsW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfTabsW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sig_prefsChanged(); break;
        case 1: _t->acceptChanges(); break;
        case 2: _t->rejectChanges(); break;
        case 3: _t->buttonClicked((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 4: _t->reloadPanels(); break;
        case 5: _t->setCurrentIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ConfTabsW::*)()>(_a, &ConfTabsW::sig_prefsChanged, 0))
            return;
    }
}

const QMetaObject *confgui::ConfTabsW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfTabsW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui9ConfTabsWE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int confgui::ConfTabsW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void confgui::ConfTabsW::sig_prefsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN7confgui10ConfPanelWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfPanelW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui10ConfPanelWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfPanelW"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfPanelW, qt_meta_tag_ZN7confgui10ConfPanelWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfPanelW::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui10ConfPanelWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui10ConfPanelWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui10ConfPanelWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfPanelW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfPanelW *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *confgui::ConfPanelW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfPanelW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui10ConfPanelWE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int confgui::ConfPanelW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN7confgui10ConfParamWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfParamW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui10ConfParamWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfParamW",
        "setEnabled",
        "",
        "storeValue",
        "setValue",
        "newvalue"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setEnabled'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'storeValue'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setValue'
        QtMocHelpers::SlotData<void(const QString &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'setValue'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'setValue'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfParamW, qt_meta_tag_ZN7confgui10ConfParamWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfParamW::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui10ConfParamWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui10ConfParamWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui10ConfParamWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfParamW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfParamW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->storeValue(); break;
        case 2: _t->setValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->setValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setValue((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *confgui::ConfParamW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfParamW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui10ConfParamWE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int confgui::ConfParamW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7confgui14ConfParamBoolWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfParamBoolW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui14ConfParamBoolWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfParamBoolW",
        "setEnabled",
        "",
        "i"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setEnabled'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfParamBoolW, qt_meta_tag_ZN7confgui14ConfParamBoolWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfParamBoolW::staticMetaObject = { {
    QMetaObject::SuperData::link<ConfParamW::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui14ConfParamBoolWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui14ConfParamBoolWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui14ConfParamBoolWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfParamBoolW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfParamBoolW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *confgui::ConfParamBoolW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfParamBoolW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui14ConfParamBoolWE_t>.strings))
        return static_cast<void*>(this);
    return ConfParamW::qt_metacast(_clname);
}

int confgui::ConfParamBoolW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ConfParamW::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7confgui13ConfParamIntWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfParamIntW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui13ConfParamIntWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfParamIntW",
        "setEnabled",
        "",
        "i"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setEnabled'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfParamIntW, qt_meta_tag_ZN7confgui13ConfParamIntWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfParamIntW::staticMetaObject = { {
    QMetaObject::SuperData::link<ConfParamW::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamIntWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamIntWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui13ConfParamIntWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfParamIntW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfParamIntW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *confgui::ConfParamIntW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfParamIntW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamIntWE_t>.strings))
        return static_cast<void*>(this);
    return ConfParamW::qt_metacast(_clname);
}

int confgui::ConfParamIntW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ConfParamW::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7confgui13ConfParamStrWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfParamStrW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui13ConfParamStrWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfParamStrW",
        "setEnabled",
        "",
        "i"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setEnabled'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfParamStrW, qt_meta_tag_ZN7confgui13ConfParamStrWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfParamStrW::staticMetaObject = { {
    QMetaObject::SuperData::link<ConfParamW::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamStrWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamStrWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui13ConfParamStrWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfParamStrW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfParamStrW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *confgui::ConfParamStrW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfParamStrW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamStrWE_t>.strings))
        return static_cast<void*>(this);
    return ConfParamW::qt_metacast(_clname);
}

int confgui::ConfParamStrW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ConfParamW::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7confgui14ConfParamCStrWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfParamCStrW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui14ConfParamCStrWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfParamCStrW",
        "setEnabled",
        "",
        "i"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setEnabled'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfParamCStrW, qt_meta_tag_ZN7confgui14ConfParamCStrWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfParamCStrW::staticMetaObject = { {
    QMetaObject::SuperData::link<ConfParamW::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui14ConfParamCStrWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui14ConfParamCStrWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui14ConfParamCStrWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfParamCStrW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfParamCStrW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *confgui::ConfParamCStrW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfParamCStrW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui14ConfParamCStrWE_t>.strings))
        return static_cast<void*>(this);
    return ConfParamW::qt_metacast(_clname);
}

int confgui::ConfParamCStrW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ConfParamW::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7confgui12ConfParamFNWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfParamFNW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui12ConfParamFNWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfParamFNW",
        "showBrowserDialog",
        "",
        "setEnabled",
        "i"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'showBrowserDialog'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setEnabled'
        QtMocHelpers::SlotData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfParamFNW, qt_meta_tag_ZN7confgui12ConfParamFNWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfParamFNW::staticMetaObject = { {
    QMetaObject::SuperData::link<ConfParamW::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui12ConfParamFNWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui12ConfParamFNWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui12ConfParamFNWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfParamFNW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfParamFNW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showBrowserDialog(); break;
        case 1: _t->setEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *confgui::ConfParamFNW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfParamFNW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui12ConfParamFNWE_t>.strings))
        return static_cast<void*>(this);
    return ConfParamW::qt_metacast(_clname);
}

int confgui::ConfParamFNW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ConfParamW::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7confgui12ConfParamSLWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfParamSLW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui12ConfParamSLWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfParamSLW",
        "entryDeleted",
        "",
        "currentTextChanged",
        "setEnabled",
        "i",
        "showInputDialog",
        "deleteSelected",
        "editSelected",
        "performInsert"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'entryDeleted'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'currentTextChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'setEnabled'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'showInputDialog'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'deleteSelected'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editSelected'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'performInsert'
        QtMocHelpers::SlotData<void(const QString &)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfParamSLW, qt_meta_tag_ZN7confgui12ConfParamSLWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfParamSLW::staticMetaObject = { {
    QMetaObject::SuperData::link<ConfParamW::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui12ConfParamSLWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui12ConfParamSLWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui12ConfParamSLWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfParamSLW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfParamSLW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->entryDeleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->currentTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->setEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->showInputDialog(); break;
        case 4: _t->deleteSelected(); break;
        case 5: _t->editSelected(); break;
        case 6: _t->performInsert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ConfParamSLW::*)(QString )>(_a, &ConfParamSLW::entryDeleted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConfParamSLW::*)(const QString & )>(_a, &ConfParamSLW::currentTextChanged, 1))
            return;
    }
}

const QMetaObject *confgui::ConfParamSLW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfParamSLW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui12ConfParamSLWE_t>.strings))
        return static_cast<void*>(this);
    return ConfParamW::qt_metacast(_clname);
}

int confgui::ConfParamSLW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ConfParamW::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void confgui::ConfParamSLW::entryDeleted(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void confgui::ConfParamSLW::currentTextChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN7confgui13ConfParamDNLWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfParamDNLW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui13ConfParamDNLWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfParamDNLW",
        "showInputDialog",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'showInputDialog'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfParamDNLW, qt_meta_tag_ZN7confgui13ConfParamDNLWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfParamDNLW::staticMetaObject = { {
    QMetaObject::SuperData::link<ConfParamSLW::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamDNLWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamDNLWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui13ConfParamDNLWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfParamDNLW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfParamDNLW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showInputDialog(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *confgui::ConfParamDNLW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfParamDNLW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamDNLWE_t>.strings))
        return static_cast<void*>(this);
    return ConfParamSLW::qt_metacast(_clname);
}

int confgui::ConfParamDNLW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ConfParamSLW::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7confgui13ConfParamCSLWE_t {};
} // unnamed namespace

template <> constexpr inline auto confgui::ConfParamCSLW::qt_create_metaobjectdata<qt_meta_tag_ZN7confgui13ConfParamCSLWE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "confgui::ConfParamCSLW",
        "showInputDialog",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'showInputDialog'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfParamCSLW, qt_meta_tag_ZN7confgui13ConfParamCSLWE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject confgui::ConfParamCSLW::staticMetaObject = { {
    QMetaObject::SuperData::link<ConfParamSLW::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamCSLWE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamCSLWE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7confgui13ConfParamCSLWE_t>.metaTypes,
    nullptr
} };

void confgui::ConfParamCSLW::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfParamCSLW *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showInputDialog(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *confgui::ConfParamCSLW::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *confgui::ConfParamCSLW::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7confgui13ConfParamCSLWE_t>.strings))
        return static_cast<void*>(this);
    return ConfParamSLW::qt_metacast(_clname);
}

int confgui::ConfParamCSLW::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ConfParamSLW::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
