/********************************************************************************
** Form generated from reading UI file 'specialindex.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SPECIALINDEX_H
#define UI_SPECIALINDEX_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_SpecIdxW
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QCheckBox *retryFailedCB;
    QCheckBox *eraseBeforeCB;
    QGridLayout *gridLayout;
    QLineEdit *targLE;
    QPushButton *targBrowsePB;
    QLabel *textLabel1;
    QHBoxLayout *horizontalLayout;
    QLabel *patLBL;
    QLineEdit *selPatsLE;
    QGridLayout *gridLayout1;
    QLineEdit *diagsLE;
    QPushButton *diagsBrowsePB;
    QLabel *textLabel2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SpecIdxW)
    {
        if (SpecIdxW->objectName().isEmpty())
            SpecIdxW->setObjectName("SpecIdxW");
        SpecIdxW->setWindowModality(Qt::WindowModal);
        SpecIdxW->resize(610, 192);
        verticalLayout_2 = new QVBoxLayout(SpecIdxW);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        retryFailedCB = new QCheckBox(SpecIdxW);
        retryFailedCB->setObjectName("retryFailedCB");

        verticalLayout->addWidget(retryFailedCB);

        eraseBeforeCB = new QCheckBox(SpecIdxW);
        eraseBeforeCB->setObjectName("eraseBeforeCB");

        verticalLayout->addWidget(eraseBeforeCB);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        targLE = new QLineEdit(SpecIdxW);
        targLE->setObjectName("targLE");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(8);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(targLE->sizePolicy().hasHeightForWidth());
        targLE->setSizePolicy(sizePolicy);
        targLE->setMinimumSize(QSize(300, 0));

        gridLayout->addWidget(targLE, 1, 0, 1, 1);

        targBrowsePB = new QPushButton(SpecIdxW);
        targBrowsePB->setObjectName("targBrowsePB");
        targBrowsePB->setAutoDefault(false);

        gridLayout->addWidget(targBrowsePB, 1, 1, 1, 1);

        textLabel1 = new QLabel(SpecIdxW);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);


        verticalLayout->addLayout(gridLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        patLBL = new QLabel(SpecIdxW);
        patLBL->setObjectName("patLBL");

        horizontalLayout->addWidget(patLBL);

        selPatsLE = new QLineEdit(SpecIdxW);
        selPatsLE->setObjectName("selPatsLE");

        horizontalLayout->addWidget(selPatsLE);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_2->addLayout(verticalLayout);

        gridLayout1 = new QGridLayout();
        gridLayout1->setObjectName("gridLayout1");
        diagsLE = new QLineEdit(SpecIdxW);
        diagsLE->setObjectName("diagsLE");
        sizePolicy.setHeightForWidth(diagsLE->sizePolicy().hasHeightForWidth());
        diagsLE->setSizePolicy(sizePolicy);
        diagsLE->setMinimumSize(QSize(300, 0));

        gridLayout1->addWidget(diagsLE, 1, 0, 1, 1);

        diagsBrowsePB = new QPushButton(SpecIdxW);
        diagsBrowsePB->setObjectName("diagsBrowsePB");
        diagsBrowsePB->setAutoDefault(false);

        gridLayout1->addWidget(diagsBrowsePB, 1, 1, 1, 1);

        textLabel2 = new QLabel(SpecIdxW);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout1->addWidget(textLabel2, 0, 0, 1, 1);


        verticalLayout_2->addLayout(gridLayout1);

        buttonBox = new QDialogButtonBox(SpecIdxW);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(SpecIdxW);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, SpecIdxW, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, SpecIdxW, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(SpecIdxW);
    } // setupUi

    void retranslateUi(QDialog *SpecIdxW)
    {
        SpecIdxW->setWindowTitle(QCoreApplication::translate("SpecIdxW", "Special Indexing", nullptr));
        retryFailedCB->setText(QCoreApplication::translate("SpecIdxW", "Retry previously failed files.", nullptr));
#if QT_CONFIG(tooltip)
        eraseBeforeCB->setToolTip(QCoreApplication::translate("SpecIdxW", "Else only modified or failed files will be processed.", nullptr));
#endif // QT_CONFIG(tooltip)
        eraseBeforeCB->setText(QCoreApplication::translate("SpecIdxW", "Erase selected files data before indexing.", nullptr));
#if QT_CONFIG(tooltip)
        targLE->setToolTip(QCoreApplication::translate("SpecIdxW", "Directory to recursively index. This must be inside the regular indexed area<br> as defined in the configuration file (topdirs).", nullptr));
#endif // QT_CONFIG(tooltip)
        targBrowsePB->setText(QCoreApplication::translate("SpecIdxW", "Browse", nullptr));
        textLabel1->setText(QCoreApplication::translate("SpecIdxW", "Start directory. Must be part of the indexed tree. Use full indexed area if empty.", nullptr));
#if QT_CONFIG(tooltip)
        patLBL->setToolTip(QCoreApplication::translate("SpecIdxW", "Leave empty to select all files. You can use multiple space-separated shell-type patterns.<br>Patterns with embedded spaces should be quoted with double quotes.<br>Can only be used if the start target is set.", nullptr));
#endif // QT_CONFIG(tooltip)
        patLBL->setText(QCoreApplication::translate("SpecIdxW", "Selection patterns:", nullptr));
        diagsBrowsePB->setText(QCoreApplication::translate("SpecIdxW", "Browse", nullptr));
        textLabel2->setText(QCoreApplication::translate("SpecIdxW", "Diagnostics output file. Will be truncated and receive indexing diagnostics (reasons for files not being indexed).", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SpecIdxW: public Ui_SpecIdxW {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SPECIALINDEX_H
