#ifndef QXTCONFIRMATIONMESSAGE_H
/****************************************************************************
* Copyright (c) 2006 - 2011, the LibQxt project.
* See the Qxt AUTHORS file for a list of authors and copyright holders.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the LibQxt project nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
* OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
* DAMAGE.
*
* <http://libqxt.org>  <foundation@libqxt.org>
*****************************************************************************/

#define QXTCONFIRMATIONMESSAGE_H

#include <QMessageBox>
#include "qxtglobal.h"

class QxtConfirmationMessagePrivate;

class QXT_GUI_EXPORT QxtConfirmationMessage : public QMessageBox
{
    Q_OBJECT
    QXT_DECLARE_PRIVATE(QxtConfirmationMessage)
    Q_PROPERTY(QString confirmationText READ confirmationText WRITE setConfirmationText)
    Q_PROPERTY(QString overrideSettingsKey READ overrideSettingsKey WRITE setOverrideSettingsKey)
    Q_PROPERTY(bool rememberOnReject READ rememberOnReject WRITE setRememberOnReject)

public:
    explicit QxtConfirmationMessage(QWidget* parent = 0);
    virtual ~QxtConfirmationMessage();

    QxtConfirmationMessage(
        QMessageBox::Icon icon, const QString& title, const QString& text,
        const QString& confirmation = QString(),
        QMessageBox::StandardButtons buttons = QMessageBox::NoButton,
        QWidget* parent = 0,
        Qt::WindowFlags flags = Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint);

    static QMessageBox::StandardButton confirm(
        QWidget* parent, const QString& title, const QString& text,
        const QString& confirmation = QString(),
        QMessageBox::StandardButtons buttons= QMessageBox::Yes | QMessageBox::No,
        QMessageBox::StandardButton defaultButton = QMessageBox::NoButton);

    QString confirmationText() const;
    void setConfirmationText(const QString& confirmation);

    QString overrideSettingsKey() const;
    void setOverrideSettingsKey(const QString& key);

    bool rememberOnReject() const;
    void setRememberOnReject(bool remember);

public Q_SLOTS:
    int exec();
    void reset();
    void onButtonClicked(QAbstractButton *button);
};

#endif // QXTCONFIRMATIONMESSAGE_H
