/********************************************************************************
** Form generated from reading UI file 'restable.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESTABLE_H
#define UI_RESTABLE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTableView>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ResTable
{
public:
    QVBoxLayout *verticalLayout;
    QSplitter *splitter;
    QTableView *tableView;
    QTextBrowser *textBrowser;

    void setupUi(QWidget *ResTable)
    {
        if (ResTable->objectName().isEmpty())
            ResTable->setObjectName("ResTable");
        ResTable->resize(640, 480);
        verticalLayout = new QVBoxLayout(ResTable);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        splitter = new QSplitter(ResTable);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Vertical);
        tableView = new QTableView(splitter);
        tableView->setObjectName("tableView");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(2);
        sizePolicy.setHeightForWidth(tableView->sizePolicy().hasHeightForWidth());
        tableView->setSizePolicy(sizePolicy);
        tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        tableView->setAlternatingRowColors(false);
        tableView->setShowGrid(true);
        tableView->setSortingEnabled(true);
        tableView->setWordWrap(false);
        tableView->setCornerButtonEnabled(false);
        splitter->addWidget(tableView);
        tableView->verticalHeader()->setVisible(false);
        textBrowser = new QTextBrowser(splitter);
        textBrowser->setObjectName("textBrowser");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textBrowser->sizePolicy().hasHeightForWidth());
        textBrowser->setSizePolicy(sizePolicy1);
        splitter->addWidget(textBrowser);

        verticalLayout->addWidget(splitter);


        retranslateUi(ResTable);

        QMetaObject::connectSlotsByName(ResTable);
    } // setupUi

    void retranslateUi(QWidget *ResTable)
    {
        (void)ResTable;
    } // retranslateUi

};

namespace Ui {
    class ResTable: public Ui_ResTable {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RESTABLE_H
