/********************************************************************************
** Form generated from reading UI file 'rclmain.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RCLMAIN_H
#define UI_RCLMAIN_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "reslist.h"
#include "ssearch_w.h"

QT_BEGIN_NAMESPACE

class Ui_RclMainBase
{
public:
    QAction *fileExitAction;
    QAction *fileToggleIndexingAction;
    QAction *fileBumpIndexingAction;
    QAction *fileStartMonitorAction;
    QAction *fileRebuildIndexAction;
    QAction *fileEraseDocHistoryAction;
    QAction *fileEraseSearchHistoryAction;
    QAction *fileExportSSearchHistoryAction;
    QAction *showMissingHelpers_Action;
    QAction *showActiveTypes_Action;
    QAction *helpAbout_RecollAction;
    QAction *checkWinVersion_Action;
    QAction *userManualAction;
    QAction *onlineManualAction;
    QAction *qlCheatSheetAction;
    QAction *toolsDoc_HistoryAction;
    QAction *toolsAdvanced_SearchAction;
    QAction *toolsSort_parametersAction;
    QAction *toolsSpellAction;
    QAction *nextPageAction;
    QAction *firstPageAction;
    QAction *prevPageAction;
    QAction *indexConfigAction;
    QAction *pTransAction;
    QAction *indexScheduleAction;
    QAction *queryPrefsAction;
    QAction *extIdxAction;
    QAction *enbSynAction;
    QAction *toggleFullScreenAction;
    QAction *zoomInAction;
    QAction *zoomOutAction;
    QAction *actionSortByDateAsc;
    QAction *actionSortByDateDesc;
    QAction *actionShowQueryDetails;
    QAction *actionShowResultsAsTable;
    QAction *actionSaveResultsAsCSV;
    QAction *actionNext_Page;
    QAction *actionPrevious_Page;
    QAction *actionFirst_Page;
    QAction *actionQuery_Fragments;
    QAction *fileRetryFailedAction;
    QAction *actionSave_last_query;
    QAction *actionLoad_saved_query;
    QAction *actionSpecial_Indexing;
    QAction *actionSwitch_Config;
    QAction *actionShow_index_statistics;
    QAction *actionWebcache_Editor;
    QWidget *centralwidget;
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    SSearch *sSearch;
    QSplitter *sideFiltersSPLT;
    QFrame *frame;
    QVBoxLayout *verticalLayoutsidefilt;
    QLabel *sideFilterLBL;
    QFrame *line2;
    QCheckBox *dateFilterCB;
    QHBoxLayout *hLayoutDatesFilter;
    QDateEdit *minDateFilterDTEDT;
    QDateEdit *maxDateFilterDTEDT;
    QCheckBox *birthDateFilterCB;
    QHBoxLayout *hLayoutDatesFilter_2;
    QDateEdit *minBirthDateFilterDTEDT;
    QDateEdit *maxBirthDateFilterDTEDT;
    QFrame *line3;
    QCheckBox *sizeFilterCB;
    QHBoxLayout *hLayoutSizesFilter;
    QSpinBox *minSizeFilterSB;
    QComboBox *minSizeFilterCMB;
    QSpinBox *maxSizeFilterSB;
    QComboBox *maxSizeFilterCMB;
    QTreeView *idxTreeView;
    QWidget *resultsLayoutWidget;
    QHBoxLayout *resultsHLayout;
    ResList *reslist;
    QStatusBar *statusbar;
    QMenuBar *MenuBar;

    void setupUi(QMainWindow *RclMainBase)
    {
        if (RclMainBase->objectName().isEmpty())
            RclMainBase->setObjectName("RclMainBase");
        RclMainBase->resize(800, 600);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(RclMainBase->sizePolicy().hasHeightForWidth());
        RclMainBase->setSizePolicy(sizePolicy);
        fileExitAction = new QAction(RclMainBase);
        fileExitAction->setObjectName("fileExitAction");
        fileExitAction->setProperty("name", QVariant(QByteArray("fileExitAction")));
        fileToggleIndexingAction = new QAction(RclMainBase);
        fileToggleIndexingAction->setObjectName("fileToggleIndexingAction");
        fileToggleIndexingAction->setProperty("name", QVariant(QByteArray("fileToggleIndexingAction")));
        fileBumpIndexingAction = new QAction(RclMainBase);
        fileBumpIndexingAction->setObjectName("fileBumpIndexingAction");
        fileBumpIndexingAction->setEnabled(false);
        fileBumpIndexingAction->setProperty("name", QVariant(QByteArray("fileBumpIndexingAction")));
        fileStartMonitorAction = new QAction(RclMainBase);
        fileStartMonitorAction->setObjectName("fileStartMonitorAction");
        fileStartMonitorAction->setEnabled(true);
        fileStartMonitorAction->setProperty("name", QVariant(QByteArray("fileStartMonitorAction")));
        fileRebuildIndexAction = new QAction(RclMainBase);
        fileRebuildIndexAction->setObjectName("fileRebuildIndexAction");
        fileRebuildIndexAction->setProperty("name", QVariant(QByteArray("fileRebuildIndexAction")));
        fileEraseDocHistoryAction = new QAction(RclMainBase);
        fileEraseDocHistoryAction->setObjectName("fileEraseDocHistoryAction");
        fileEraseDocHistoryAction->setProperty("name", QVariant(QByteArray("fileEraseDocHistoryAction")));
        fileEraseSearchHistoryAction = new QAction(RclMainBase);
        fileEraseSearchHistoryAction->setObjectName("fileEraseSearchHistoryAction");
        fileEraseSearchHistoryAction->setProperty("name", QVariant(QByteArray("fileEraseSearchHistoryAction")));
        fileExportSSearchHistoryAction = new QAction(RclMainBase);
        fileExportSSearchHistoryAction->setObjectName("fileExportSSearchHistoryAction");
        fileExportSSearchHistoryAction->setProperty("name", QVariant(QByteArray("fileExportSSearchHistoryAction")));
        showMissingHelpers_Action = new QAction(RclMainBase);
        showMissingHelpers_Action->setObjectName("showMissingHelpers_Action");
        showMissingHelpers_Action->setProperty("name", QVariant(QByteArray("showMissingHelpers_Action")));
        showActiveTypes_Action = new QAction(RclMainBase);
        showActiveTypes_Action->setObjectName("showActiveTypes_Action");
        showActiveTypes_Action->setProperty("name", QVariant(QByteArray("showActiveTypes_Action")));
        helpAbout_RecollAction = new QAction(RclMainBase);
        helpAbout_RecollAction->setObjectName("helpAbout_RecollAction");
        helpAbout_RecollAction->setProperty("name", QVariant(QByteArray("helpAbout_RecollAction")));
        checkWinVersion_Action = new QAction(RclMainBase);
        checkWinVersion_Action->setObjectName("checkWinVersion_Action");
        checkWinVersion_Action->setProperty("name", QVariant(QByteArray("checkWinVersion_Action")));
        userManualAction = new QAction(RclMainBase);
        userManualAction->setObjectName("userManualAction");
        userManualAction->setProperty("name", QVariant(QByteArray("userManualAction")));
        onlineManualAction = new QAction(RclMainBase);
        onlineManualAction->setObjectName("onlineManualAction");
        onlineManualAction->setProperty("name", QVariant(QByteArray("onlineManualAction")));
        qlCheatSheetAction = new QAction(RclMainBase);
        qlCheatSheetAction->setObjectName("qlCheatSheetAction");
        qlCheatSheetAction->setProperty("name", QVariant(QByteArray("qlCheatSheetAction")));
        toolsDoc_HistoryAction = new QAction(RclMainBase);
        toolsDoc_HistoryAction->setObjectName("toolsDoc_HistoryAction");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/history.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        toolsDoc_HistoryAction->setIcon(icon);
        toolsDoc_HistoryAction->setProperty("name", QVariant(QByteArray("toolsDoc_HistoryAction")));
        toolsAdvanced_SearchAction = new QAction(RclMainBase);
        toolsAdvanced_SearchAction->setObjectName("toolsAdvanced_SearchAction");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/asearch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        toolsAdvanced_SearchAction->setIcon(icon1);
        toolsAdvanced_SearchAction->setProperty("name", QVariant(QByteArray("toolsAdvanced_SearchAction")));
        toolsSort_parametersAction = new QAction(RclMainBase);
        toolsSort_parametersAction->setObjectName("toolsSort_parametersAction");
        toolsSort_parametersAction->setProperty("name", QVariant(QByteArray("toolsSort_parametersAction")));
        toolsSpellAction = new QAction(RclMainBase);
        toolsSpellAction->setObjectName("toolsSpellAction");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/images/spell.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        toolsSpellAction->setIcon(icon2);
        toolsSpellAction->setProperty("name", QVariant(QByteArray("toolsSpellAction")));
        nextPageAction = new QAction(RclMainBase);
        nextPageAction->setObjectName("nextPageAction");
        nextPageAction->setEnabled(false);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/images/nextpage.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        nextPageAction->setIcon(icon3);
        nextPageAction->setProperty("name", QVariant(QByteArray("nextPageAction")));
        firstPageAction = new QAction(RclMainBase);
        firstPageAction->setObjectName("firstPageAction");
        firstPageAction->setEnabled(false);
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/images/firstpage.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        firstPageAction->setIcon(icon4);
        firstPageAction->setProperty("name", QVariant(QByteArray("firstPageAction")));
        prevPageAction = new QAction(RclMainBase);
        prevPageAction->setObjectName("prevPageAction");
        prevPageAction->setEnabled(false);
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/images/prevpage.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        prevPageAction->setIcon(icon5);
        prevPageAction->setProperty("name", QVariant(QByteArray("prevPageAction")));
        indexConfigAction = new QAction(RclMainBase);
        indexConfigAction->setObjectName("indexConfigAction");
        indexConfigAction->setProperty("name", QVariant(QByteArray("indexConfigAction")));
        pTransAction = new QAction(RclMainBase);
        pTransAction->setObjectName("pTransAction");
        pTransAction->setProperty("name", QVariant(QByteArray("pTransAction")));
        indexScheduleAction = new QAction(RclMainBase);
        indexScheduleAction->setObjectName("indexScheduleAction");
        indexScheduleAction->setProperty("name", QVariant(QByteArray("indexScheduleAction")));
        queryPrefsAction = new QAction(RclMainBase);
        queryPrefsAction->setObjectName("queryPrefsAction");
        queryPrefsAction->setProperty("name", QVariant(QByteArray("queryPrefsAction")));
        extIdxAction = new QAction(RclMainBase);
        extIdxAction->setObjectName("extIdxAction");
        extIdxAction->setProperty("name", QVariant(QByteArray("extIdxAction")));
        enbSynAction = new QAction(RclMainBase);
        enbSynAction->setObjectName("enbSynAction");
        enbSynAction->setCheckable(true);
        enbSynAction->setProperty("name", QVariant(QByteArray("enbSynAction")));
        toggleFullScreenAction = new QAction(RclMainBase);
        toggleFullScreenAction->setObjectName("toggleFullScreenAction");
        toggleFullScreenAction->setProperty("name", QVariant(QByteArray("toggleFullScreenAction")));
        zoomInAction = new QAction(RclMainBase);
        zoomInAction->setObjectName("zoomInAction");
        zoomInAction->setProperty("name", QVariant(QByteArray("zoomInAction")));
        zoomOutAction = new QAction(RclMainBase);
        zoomOutAction->setObjectName("zoomOutAction");
        zoomOutAction->setProperty("name", QVariant(QByteArray("zoomOutAction")));
        actionSortByDateAsc = new QAction(RclMainBase);
        actionSortByDateAsc->setObjectName("actionSortByDateAsc");
        actionSortByDateAsc->setCheckable(true);
        actionSortByDateAsc->setEnabled(false);
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/images/up.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        actionSortByDateAsc->setIcon(icon6);
        actionSortByDateDesc = new QAction(RclMainBase);
        actionSortByDateDesc->setObjectName("actionSortByDateDesc");
        actionSortByDateDesc->setCheckable(true);
        actionSortByDateDesc->setEnabled(false);
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/images/down.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        actionSortByDateDesc->setIcon(icon7);
        actionShowQueryDetails = new QAction(RclMainBase);
        actionShowQueryDetails->setObjectName("actionShowQueryDetails");
        actionShowResultsAsTable = new QAction(RclMainBase);
        actionShowResultsAsTable->setObjectName("actionShowResultsAsTable");
        actionShowResultsAsTable->setCheckable(true);
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/images/table.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        actionShowResultsAsTable->setIcon(icon8);
        actionSaveResultsAsCSV = new QAction(RclMainBase);
        actionSaveResultsAsCSV->setObjectName("actionSaveResultsAsCSV");
        actionNext_Page = new QAction(RclMainBase);
        actionNext_Page->setObjectName("actionNext_Page");
        actionPrevious_Page = new QAction(RclMainBase);
        actionPrevious_Page->setObjectName("actionPrevious_Page");
        actionFirst_Page = new QAction(RclMainBase);
        actionFirst_Page->setObjectName("actionFirst_Page");
        actionQuery_Fragments = new QAction(RclMainBase);
        actionQuery_Fragments->setObjectName("actionQuery_Fragments");
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/images/code-block.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        actionQuery_Fragments->setIcon(icon9);
        fileRetryFailedAction = new QAction(RclMainBase);
        fileRetryFailedAction->setObjectName("fileRetryFailedAction");
        fileRetryFailedAction->setCheckable(true);
        fileRetryFailedAction->setProperty("name", QVariant(QByteArray("fileToggleIndexingAction")));
        actionSave_last_query = new QAction(RclMainBase);
        actionSave_last_query->setObjectName("actionSave_last_query");
        actionLoad_saved_query = new QAction(RclMainBase);
        actionLoad_saved_query->setObjectName("actionLoad_saved_query");
        actionSpecial_Indexing = new QAction(RclMainBase);
        actionSpecial_Indexing->setObjectName("actionSpecial_Indexing");
        actionSwitch_Config = new QAction(RclMainBase);
        actionSwitch_Config->setObjectName("actionSwitch_Config");
        actionShow_index_statistics = new QAction(RclMainBase);
        actionShow_index_statistics->setObjectName("actionShow_index_statistics");
        actionWebcache_Editor = new QAction(RclMainBase);
        actionWebcache_Editor->setObjectName("actionWebcache_Editor");
        centralwidget = new QWidget(RclMainBase);
        centralwidget->setObjectName("centralwidget");
        gridLayout = new QGridLayout(centralwidget);
        gridLayout->setSpacing(2);
        gridLayout->setContentsMargins(2, 2, 2, 2);
        gridLayout->setObjectName("gridLayout");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(2);
        verticalLayout->setObjectName("verticalLayout");
        sSearch = new SSearch(centralwidget);
        sSearch->setObjectName("sSearch");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(sSearch->sizePolicy().hasHeightForWidth());
        sSearch->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(sSearch);

        sideFiltersSPLT = new QSplitter(centralwidget);
        sideFiltersSPLT->setObjectName("sideFiltersSPLT");
        sideFiltersSPLT->setOrientation(Qt::Horizontal);
        frame = new QFrame(sideFiltersSPLT);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        verticalLayoutsidefilt = new QVBoxLayout(frame);
        verticalLayoutsidefilt->setSpacing(2);
        verticalLayoutsidefilt->setContentsMargins(2, 2, 2, 2);
        verticalLayoutsidefilt->setObjectName("verticalLayoutsidefilt");
        sideFilterLBL = new QLabel(frame);
        sideFilterLBL->setObjectName("sideFilterLBL");

        verticalLayoutsidefilt->addWidget(sideFilterLBL);

        line2 = new QFrame(frame);
        line2->setObjectName("line2");
        line2->setFrameShape(QFrame::Shape::HLine);
        line2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayoutsidefilt->addWidget(line2);

        dateFilterCB = new QCheckBox(frame);
        dateFilterCB->setObjectName("dateFilterCB");
        dateFilterCB->setChecked(false);

        verticalLayoutsidefilt->addWidget(dateFilterCB);

        hLayoutDatesFilter = new QHBoxLayout();
        hLayoutDatesFilter->setSpacing(2);
        hLayoutDatesFilter->setObjectName("hLayoutDatesFilter");
        minDateFilterDTEDT = new QDateEdit(frame);
        minDateFilterDTEDT->setObjectName("minDateFilterDTEDT");
        minDateFilterDTEDT->setEnabled(false);

        hLayoutDatesFilter->addWidget(minDateFilterDTEDT);

        maxDateFilterDTEDT = new QDateEdit(frame);
        maxDateFilterDTEDT->setObjectName("maxDateFilterDTEDT");
        maxDateFilterDTEDT->setEnabled(false);

        hLayoutDatesFilter->addWidget(maxDateFilterDTEDT);


        verticalLayoutsidefilt->addLayout(hLayoutDatesFilter);

        birthDateFilterCB = new QCheckBox(frame);
        birthDateFilterCB->setObjectName("birthDateFilterCB");
        birthDateFilterCB->setChecked(false);

        verticalLayoutsidefilt->addWidget(birthDateFilterCB);

        hLayoutDatesFilter_2 = new QHBoxLayout();
        hLayoutDatesFilter_2->setSpacing(2);
        hLayoutDatesFilter_2->setObjectName("hLayoutDatesFilter_2");
        minBirthDateFilterDTEDT = new QDateEdit(frame);
        minBirthDateFilterDTEDT->setObjectName("minBirthDateFilterDTEDT");
        minBirthDateFilterDTEDT->setEnabled(false);

        hLayoutDatesFilter_2->addWidget(minBirthDateFilterDTEDT);

        maxBirthDateFilterDTEDT = new QDateEdit(frame);
        maxBirthDateFilterDTEDT->setObjectName("maxBirthDateFilterDTEDT");
        maxBirthDateFilterDTEDT->setEnabled(false);

        hLayoutDatesFilter_2->addWidget(maxBirthDateFilterDTEDT);


        verticalLayoutsidefilt->addLayout(hLayoutDatesFilter_2);

        line3 = new QFrame(frame);
        line3->setObjectName("line3");
        line3->setFrameShape(QFrame::Shape::HLine);
        line3->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayoutsidefilt->addWidget(line3);

        sizeFilterCB = new QCheckBox(frame);
        sizeFilterCB->setObjectName("sizeFilterCB");
        sizeFilterCB->setChecked(false);

        verticalLayoutsidefilt->addWidget(sizeFilterCB);

        hLayoutSizesFilter = new QHBoxLayout();
        hLayoutSizesFilter->setSpacing(2);
        hLayoutSizesFilter->setObjectName("hLayoutSizesFilter");
        minSizeFilterSB = new QSpinBox(frame);
        minSizeFilterSB->setObjectName("minSizeFilterSB");
        minSizeFilterSB->setEnabled(false);
        minSizeFilterSB->setMaximum(99999);

        hLayoutSizesFilter->addWidget(minSizeFilterSB);

        minSizeFilterCMB = new QComboBox(frame);
        minSizeFilterCMB->setObjectName("minSizeFilterCMB");
        minSizeFilterCMB->setEnabled(false);

        hLayoutSizesFilter->addWidget(minSizeFilterCMB);

        maxSizeFilterSB = new QSpinBox(frame);
        maxSizeFilterSB->setObjectName("maxSizeFilterSB");
        maxSizeFilterSB->setEnabled(false);
        maxSizeFilterSB->setMaximum(99999);

        hLayoutSizesFilter->addWidget(maxSizeFilterSB);

        maxSizeFilterCMB = new QComboBox(frame);
        maxSizeFilterCMB->setObjectName("maxSizeFilterCMB");
        maxSizeFilterCMB->setEnabled(false);

        hLayoutSizesFilter->addWidget(maxSizeFilterCMB);


        verticalLayoutsidefilt->addLayout(hLayoutSizesFilter);

        idxTreeView = new QTreeView(frame);
        idxTreeView->setObjectName("idxTreeView");
        idxTreeView->setFrameShape(QFrame::NoFrame);

        verticalLayoutsidefilt->addWidget(idxTreeView);

        sideFiltersSPLT->addWidget(frame);
        resultsLayoutWidget = new QWidget(sideFiltersSPLT);
        resultsLayoutWidget->setObjectName("resultsLayoutWidget");
        resultsHLayout = new QHBoxLayout(resultsLayoutWidget);
        resultsHLayout->setSpacing(2);
        resultsHLayout->setContentsMargins(2, 2, 2, 2);
        resultsHLayout->setObjectName("resultsHLayout");
        resultsHLayout->setContentsMargins(0, 0, 0, 0);
        reslist = new ResList(resultsLayoutWidget);
        reslist->setObjectName("reslist");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(2);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(reslist->sizePolicy().hasHeightForWidth());
        reslist->setSizePolicy(sizePolicy2);

        resultsHLayout->addWidget(reslist);

        sideFiltersSPLT->addWidget(resultsLayoutWidget);

        verticalLayout->addWidget(sideFiltersSPLT);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 1);

        RclMainBase->setCentralWidget(centralwidget);
        statusbar = new QStatusBar(RclMainBase);
        statusbar->setObjectName("statusbar");
        RclMainBase->setStatusBar(statusbar);
        MenuBar = new QMenuBar(RclMainBase);
        MenuBar->setObjectName("MenuBar");
        MenuBar->setGeometry(QRect(0, 0, 800, 23));
        RclMainBase->setMenuBar(MenuBar);

        MenuBar->addSeparator();

        retranslateUi(RclMainBase);

        QMetaObject::connectSlotsByName(RclMainBase);
    } // setupUi

    void retranslateUi(QMainWindow *RclMainBase)
    {
        RclMainBase->setWindowTitle(QCoreApplication::translate("RclMainBase", "Recoll", nullptr));
        fileExitAction->setText(QCoreApplication::translate("RclMainBase", "E&xit", nullptr));
#if QT_CONFIG(shortcut)
        fileExitAction->setShortcut(QCoreApplication::translate("RclMainBase", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        fileToggleIndexingAction->setText(QCoreApplication::translate("RclMainBase", "Update &index", nullptr));
        fileBumpIndexingAction->setText(QCoreApplication::translate("RclMainBase", "Trigger incremental pass", nullptr));
        fileStartMonitorAction->setText(QCoreApplication::translate("RclMainBase", "Start real time indexer", nullptr));
        fileRebuildIndexAction->setText(QCoreApplication::translate("RclMainBase", "&Rebuild index", nullptr));
        fileEraseDocHistoryAction->setText(QCoreApplication::translate("RclMainBase", "&Erase document history", nullptr));
        fileEraseSearchHistoryAction->setText(QCoreApplication::translate("RclMainBase", "&Erase search history", nullptr));
        fileExportSSearchHistoryAction->setText(QCoreApplication::translate("RclMainBase", "E&xport simple search history", nullptr));
        showMissingHelpers_Action->setText(QCoreApplication::translate("RclMainBase", "Missing &helpers", nullptr));
        showActiveTypes_Action->setText(QCoreApplication::translate("RclMainBase", "Indexed &MIME types", nullptr));
        helpAbout_RecollAction->setText(QCoreApplication::translate("RclMainBase", "&About Recoll", nullptr));
        checkWinVersion_Action->setText(QCoreApplication::translate("RclMainBase", "Check Recoll Web site for current Windows version", nullptr));
        userManualAction->setText(QCoreApplication::translate("RclMainBase", "&User manual (local, one HTML page)", nullptr));
        onlineManualAction->setText(QCoreApplication::translate("RclMainBase", "&Online manual (Recoll Web site)", nullptr));
        qlCheatSheetAction->setText(QCoreApplication::translate("RclMainBase", "Query Language cheat sheet", nullptr));
        toolsDoc_HistoryAction->setText(QCoreApplication::translate("RclMainBase", "Document &History", nullptr));
#if QT_CONFIG(tooltip)
        toolsDoc_HistoryAction->setToolTip(QCoreApplication::translate("RclMainBase", "Document  History", nullptr));
#endif // QT_CONFIG(tooltip)
        toolsAdvanced_SearchAction->setText(QCoreApplication::translate("RclMainBase", "&Advanced Search", nullptr));
#if QT_CONFIG(tooltip)
        toolsAdvanced_SearchAction->setToolTip(QCoreApplication::translate("RclMainBase", "Assisted complex search", nullptr));
#endif // QT_CONFIG(tooltip)
        toolsSort_parametersAction->setText(QCoreApplication::translate("RclMainBase", "&Sort parameters", nullptr));
#if QT_CONFIG(tooltip)
        toolsSort_parametersAction->setToolTip(QCoreApplication::translate("RclMainBase", "Sort parameters", nullptr));
#endif // QT_CONFIG(tooltip)
        toolsSpellAction->setText(QCoreApplication::translate("RclMainBase", "Term &explorer", nullptr));
#if QT_CONFIG(tooltip)
        toolsSpellAction->setToolTip(QCoreApplication::translate("RclMainBase", "Term explorer tool", nullptr));
#endif // QT_CONFIG(tooltip)
        nextPageAction->setIconText(QCoreApplication::translate("RclMainBase", "Next page", nullptr));
#if QT_CONFIG(tooltip)
        nextPageAction->setToolTip(QCoreApplication::translate("RclMainBase", "Next page of results", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        nextPageAction->setShortcut(QCoreApplication::translate("RclMainBase", "PgDown", nullptr));
#endif // QT_CONFIG(shortcut)
        firstPageAction->setIconText(QCoreApplication::translate("RclMainBase", "First page", nullptr));
#if QT_CONFIG(tooltip)
        firstPageAction->setToolTip(QCoreApplication::translate("RclMainBase", "Go to first page of results", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        firstPageAction->setShortcut(QCoreApplication::translate("RclMainBase", "Shift+PgUp", nullptr));
#endif // QT_CONFIG(shortcut)
        prevPageAction->setIconText(QCoreApplication::translate("RclMainBase", "Previous page", nullptr));
#if QT_CONFIG(tooltip)
        prevPageAction->setToolTip(QCoreApplication::translate("RclMainBase", "Previous page of results", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        prevPageAction->setShortcut(QCoreApplication::translate("RclMainBase", "PgUp", nullptr));
#endif // QT_CONFIG(shortcut)
        indexConfigAction->setText(QCoreApplication::translate("RclMainBase", "&Index configuration", nullptr));
        pTransAction->setText(QCoreApplication::translate("RclMainBase", "Path translations", nullptr));
        indexScheduleAction->setText(QCoreApplication::translate("RclMainBase", "Indexing &schedule", nullptr));
        queryPrefsAction->setText(QCoreApplication::translate("RclMainBase", "&GUI configuration", nullptr));
        extIdxAction->setText(QCoreApplication::translate("RclMainBase", "E&xternal index dialog", nullptr));
        extIdxAction->setIconText(QCoreApplication::translate("RclMainBase", "External index dialog", nullptr));
        enbSynAction->setText(QCoreApplication::translate("RclMainBase", "Enable synonyms", nullptr));
        enbSynAction->setIconText(QCoreApplication::translate("RclMainBase", "Enable synonyms", nullptr));
        toggleFullScreenAction->setText(QCoreApplication::translate("RclMainBase", "&Full Screen", nullptr));
        toggleFullScreenAction->setIconText(QCoreApplication::translate("RclMainBase", "Full Screen", nullptr));
#if QT_CONFIG(shortcut)
        toggleFullScreenAction->setShortcut(QCoreApplication::translate("RclMainBase", "F11", nullptr));
#endif // QT_CONFIG(shortcut)
        zoomInAction->setText(QCoreApplication::translate("RclMainBase", "Increase results text font size", nullptr));
        zoomInAction->setIconText(QCoreApplication::translate("RclMainBase", "Increase Font Size", nullptr));
        zoomOutAction->setText(QCoreApplication::translate("RclMainBase", "Decrease results text font size", nullptr));
        zoomOutAction->setIconText(QCoreApplication::translate("RclMainBase", "Decrease Font Size", nullptr));
        actionSortByDateAsc->setText(QCoreApplication::translate("RclMainBase", "Sort by date, oldest first", nullptr));
#if QT_CONFIG(tooltip)
        actionSortByDateAsc->setToolTip(QCoreApplication::translate("RclMainBase", "Sort by dates from oldest to newest", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSortByDateDesc->setText(QCoreApplication::translate("RclMainBase", "Sort by date, newest first", nullptr));
#if QT_CONFIG(tooltip)
        actionSortByDateDesc->setToolTip(QCoreApplication::translate("RclMainBase", "Sort by dates from newest to oldest", nullptr));
#endif // QT_CONFIG(tooltip)
        actionShowQueryDetails->setText(QCoreApplication::translate("RclMainBase", "Show Query Details", nullptr));
        actionShowResultsAsTable->setText(QCoreApplication::translate("RclMainBase", "Show as table", nullptr));
#if QT_CONFIG(tooltip)
        actionShowResultsAsTable->setToolTip(QCoreApplication::translate("RclMainBase", "Show results in a spreadsheet-like table", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSaveResultsAsCSV->setText(QCoreApplication::translate("RclMainBase", "Save as CSV (spreadsheet) file", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveResultsAsCSV->setToolTip(QCoreApplication::translate("RclMainBase", "Saves the result into a file which you can load in a spreadsheet", nullptr));
#endif // QT_CONFIG(tooltip)
        actionNext_Page->setText(QCoreApplication::translate("RclMainBase", "Next Page", nullptr));
        actionPrevious_Page->setText(QCoreApplication::translate("RclMainBase", "Previous Page", nullptr));
        actionFirst_Page->setText(QCoreApplication::translate("RclMainBase", "First Page", nullptr));
        actionQuery_Fragments->setText(QCoreApplication::translate("RclMainBase", "Query Fragments", nullptr));
        fileRetryFailedAction->setText(QCoreApplication::translate("RclMainBase", "    With failed files retrying", nullptr));
#if QT_CONFIG(tooltip)
        fileRetryFailedAction->setToolTip(QCoreApplication::translate("RclMainBase", "Next update will retry previously failed files", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSave_last_query->setText(QCoreApplication::translate("RclMainBase", "Save last query", nullptr));
        actionLoad_saved_query->setText(QCoreApplication::translate("RclMainBase", "Load saved query", nullptr));
        actionSpecial_Indexing->setText(QCoreApplication::translate("RclMainBase", "Special Indexing", nullptr));
#if QT_CONFIG(tooltip)
        actionSpecial_Indexing->setToolTip(QCoreApplication::translate("RclMainBase", "Indexing with special options", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSwitch_Config->setText(QCoreApplication::translate("RclMainBase", "Switch Configuration...", nullptr));
#if QT_CONFIG(tooltip)
        actionSwitch_Config->setToolTip(QCoreApplication::translate("RclMainBase", "Choose another configuration to run on, replacing this process", nullptr));
#endif // QT_CONFIG(tooltip)
        actionShow_index_statistics->setText(QCoreApplication::translate("RclMainBase", "Index &statistics", nullptr));
        actionWebcache_Editor->setText(QCoreApplication::translate("RclMainBase", "Webcache Editor", nullptr));
        sideFilterLBL->setText(QCoreApplication::translate("RclMainBase", "Query Language Filters", nullptr));
        dateFilterCB->setText(QCoreApplication::translate("RclMainBase", "Filter dates", nullptr));
        birthDateFilterCB->setText(QCoreApplication::translate("RclMainBase", "Filter birth dates", nullptr));
        sizeFilterCB->setText(QCoreApplication::translate("RclMainBase", "Filter file sizes", nullptr));
#if QT_CONFIG(tooltip)
        minSizeFilterSB->setToolTip(QCoreApplication::translate("RclMainBase", "Minimum size (leave at 0 to disable)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        maxSizeFilterSB->setToolTip(QCoreApplication::translate("RclMainBase", "Maximum size (leave at 0 to disable)", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class RclMainBase: public Ui_RclMainBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RCLMAIN_H
