/********************************************************************************
** Form generated from reading UI file 'preview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREVIEW_H
#define UI_PREVIEW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Preview
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *pvTab;
    QWidget *tab;
    QHBoxLayout *horizontalLayout;
    QLabel *searchLBL;
    QComboBox *searchTextCMB;
    QPushButton *nextPB;
    QPushButton *prevPB;
    QPushButton *clearPB;
    QCheckBox *casematchCB;
    QLabel *hitIndicatorLBL;
    QSpacerItem *horizontalSpacer;
    QFrame *line;
    QPushButton *prevInTabPB;
    QPushButton *nextInTabPB;
    QFrame *line_2;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *editPB;

    void setupUi(QWidget *Preview)
    {
        if (Preview->objectName().isEmpty())
            Preview->setObjectName("Preview");
        Preview->resize(777, 432);
        verticalLayout = new QVBoxLayout(Preview);
        verticalLayout->setObjectName("verticalLayout");
        pvTab = new QTabWidget(Preview);
        pvTab->setObjectName("pvTab");
        pvTab->setTabsClosable(true);
        tab = new QWidget();
        tab->setObjectName("tab");
        pvTab->addTab(tab, QString());

        verticalLayout->addWidget(pvTab);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        searchLBL = new QLabel(Preview);
        searchLBL->setObjectName("searchLBL");

        horizontalLayout->addWidget(searchLBL);

        searchTextCMB = new QComboBox(Preview);
        searchTextCMB->setObjectName("searchTextCMB");
        searchTextCMB->setEditable(true);
        searchTextCMB->setInsertPolicy(QComboBox::NoInsert);

        horizontalLayout->addWidget(searchTextCMB);

        nextPB = new QPushButton(Preview);
        nextPB->setObjectName("nextPB");

        horizontalLayout->addWidget(nextPB);

        prevPB = new QPushButton(Preview);
        prevPB->setObjectName("prevPB");

        horizontalLayout->addWidget(prevPB);

        clearPB = new QPushButton(Preview);
        clearPB->setObjectName("clearPB");
        clearPB->setEnabled(false);

        horizontalLayout->addWidget(clearPB);

        casematchCB = new QCheckBox(Preview);
        casematchCB->setObjectName("casematchCB");

        horizontalLayout->addWidget(casematchCB);

        hitIndicatorLBL = new QLabel(Preview);
        hitIndicatorLBL->setObjectName("hitIndicatorLBL");

        horizontalLayout->addWidget(hitIndicatorLBL);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        line = new QFrame(Preview);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::VLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(line);

        prevInTabPB = new QPushButton(Preview);
        prevInTabPB->setObjectName("prevInTabPB");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/prevpage.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        prevInTabPB->setIcon(icon);

        horizontalLayout->addWidget(prevInTabPB);

        nextInTabPB = new QPushButton(Preview);
        nextInTabPB->setObjectName("nextInTabPB");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/nextpage.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        nextInTabPB->setIcon(icon1);

        horizontalLayout->addWidget(nextInTabPB);

        line_2 = new QFrame(Preview);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::VLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(line_2);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        editPB = new QPushButton(Preview);
        editPB->setObjectName("editPB");

        horizontalLayout->addWidget(editPB);


        verticalLayout->addLayout(horizontalLayout);

#if QT_CONFIG(shortcut)
        searchLBL->setBuddy(searchTextCMB);
#endif // QT_CONFIG(shortcut)

        retranslateUi(Preview);

        pvTab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Preview);
    } // setupUi

    void retranslateUi(QWidget *Preview)
    {
        Preview->setWindowTitle(QCoreApplication::translate("Preview", "Form", nullptr));
        pvTab->setTabText(pvTab->indexOf(tab), QCoreApplication::translate("Preview", "Tab 1", nullptr));
        searchLBL->setText(QCoreApplication::translate("Preview", "&Search for:", nullptr));
        nextPB->setText(QCoreApplication::translate("Preview", "&Next", nullptr));
        prevPB->setText(QCoreApplication::translate("Preview", "&Previous", nullptr));
        clearPB->setText(QCoreApplication::translate("Preview", "Clear", nullptr));
        casematchCB->setText(QCoreApplication::translate("Preview", "Match &Case", nullptr));
        hitIndicatorLBL->setText(QString());
        prevInTabPB->setText(QString());
#if QT_CONFIG(tooltip)
        prevInTabPB->setToolTip(QCoreApplication::translate("Preview", "Previous result document", nullptr));
#endif // QT_CONFIG(tooltip)
        nextInTabPB->setText(QString());
#if QT_CONFIG(tooltip)
        nextInTabPB->setToolTip(QCoreApplication::translate("Preview", "Next result document", nullptr));
#endif // QT_CONFIG(tooltip)
        editPB->setText(QCoreApplication::translate("Preview", "Open", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Preview: public Ui_Preview {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREVIEW_H
